# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Modified ShARC datatset"""


import json

import datasets


_CITATION = """\
@inproceedings{verma-etal-2020-neural,
    title = "Neural Conversational {QA}: Learning to Reason vs Exploiting Patterns",
    author = "Verma, Nikhil  and
      Sharma, Abhishek  and
      Madan, Dhiraj  and
      Contractor, Danish  and
      Kumar, Harshit  and
      Joshi, Sachindra",
    booktitle = "Proceedings of the 2020 Conference on Empirical Methods in Natural Language Processing (EMNLP)",
    month = nov,
    year = "2020",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/2020.emnlp-main.589",
    pages = "7263--7269",
    abstract = "Neural Conversational QA tasks such as ShARC require systems to answer questions based on the contents of a given passage. On studying recent state-of-the-art models on the ShARC QA task, we found indications that the model(s) learn spurious clues/patterns in the data-set. Further, a heuristic-based program, built to exploit these patterns, had comparative performance to that of the neural models. In this paper we share our findings about the four types of patterns in the ShARC corpus and how the neural models exploit them. Motivated by the above findings, we create and share a modified data-set that has fewer spurious patterns than the original data-set, consequently allowing models to learn better.",
}
"""

_DESCRIPTION = """\
ShARC, a conversational QA task, requires a system to answer user questions based on rules expressed in natural language text. \
However, it is found that in the ShARC dataset there are multiple spurious patterns that could be exploited by neural models. \
SharcModified is a new dataset which reduces the patterns identified in the original dataset. \
To reduce the sensitivity of neural models, for each occurence of an instance conforming to any of the patterns, \
we automatically construct alternatives where we choose to either replace the current instance with an alternative \
instance which does not exhibit the pattern; or retain the original instance. \
The modified ShARC has two versions sharc-mod and history-shuffled. For morre details refer to Appendix A.3 .
"""

_BASE_URL = "https://raw.githubusercontent.com/nikhilweee/neural-conv-qa/master/datasets"


class SharcModifiedConfig(datasets.BuilderConfig):
    """BuilderConfig for SharcModified."""

    def __init__(self, **kwargs):
        """BuilderConfig for MedHop.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(SharcModifiedConfig, self).__init__(**kwargs)


class SharcModified(datasets.GeneratorBasedBuilder):
    """Modified ShARC datatset"""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        SharcModifiedConfig(
            name="mod",
            version=datasets.Version("1.0.0"),
            description="The modified ShARC dataset.",
        ),
        SharcModifiedConfig(
            name="mod_dev_multi",
            version=datasets.Version("1.0.0"),
            description="The modified ShARC dev dataset with multiple references.",
        ),
        SharcModifiedConfig(
            name="history", version=datasets.Version("1.0.0"), description="History-Shuffled ShARC dataset."
        ),
        SharcModifiedConfig(
            name="history_dev_multi",
            version=datasets.Version("1.0.0"),
            description="History-Shuffled dev dataset with multiple references.",
        ),
    ]
    BUILDER_CONFIG_CLASS = SharcModifiedConfig

    def _info(self):
        features = {
            "id": datasets.Value("string"),
            "utterance_id": datasets.Value("string"),
            "source_url": datasets.Value("string"),
            "snippet": datasets.Value("string"),
            "question": datasets.Value("string"),
            "scenario": datasets.Value("string"),
            "history": [
                {"follow_up_question": datasets.Value("string"), "follow_up_answer": datasets.Value("string")}
            ],
            "evidence": [
                {"follow_up_question": datasets.Value("string"), "follow_up_answer": datasets.Value("string")}
            ],
            "answer": datasets.Value("string"),
        }

        if self.config.name in ["mod_dev_multi", "history_dev_multi"]:
            features["all_answers"] = datasets.Sequence(datasets.Value("string"))

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(features),
            supervised_keys=None,
            homepage="https://github.com/nikhilweee/neural-conv-qa",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        if self.config.name in ["mod_dev_multi", "history_dev_multi"]:
            url = f"{_BASE_URL}/{self.config.name}.json"
            downloaded_file = dl_manager.download(url)
            return [datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_file})]

        urls = {
            "train": f"{_BASE_URL}/{self.config.name}_train.json",
            "dev": f"{_BASE_URL}/{self.config.name}_dev.json",
        }
        downloaded_files = dl_manager.download(urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": downloaded_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": downloaded_files["dev"]},
            ),
        ]

    def _generate_examples(self, filepath):
        with open(filepath, encoding="utf-8") as f:
            examples = json.load(f)
            for i, example in enumerate(examples):
                example.pop("tree_id")
                example["id"] = example["utterance_id"]
                # the keys are misspelled for one of the example in dev set, fix it here
                for evidence in example["evidence"]:
                    if evidence.get("followup_answer") is not None:
                        evidence["follow_up_answer"] = evidence.pop("followup_answer")
                    if evidence.get("followup_question") is not None:
                        evidence["follow_up_question"] = evidence.pop("followup_question")
                yield example["id"], example



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""SelQA: A New Benchmark for Selection-Based Question Answering"""


import csv
import json

import datasets


# TODO: Add BibTeX citation
# Find for instance the citation on arxiv or on the dataset repo/website
_CITATION = """\
@InProceedings{7814688,
  author={T. {Jurczyk} and M. {Zhai} and J. D. {Choi}},
  booktitle={2016 IEEE 28th International Conference on Tools with Artificial Intelligence (ICTAI)},
  title={SelQA: A New Benchmark for Selection-Based Question Answering},
  year={2016},
  volume={},
  number={},
  pages={820-827},
  doi={10.1109/ICTAI.2016.0128}
}
"""

# TODO: Add description of the dataset here
# You can copy an official description
_DESCRIPTION = """\
The SelQA dataset provides crowdsourced annotation for two selection-based question answer tasks,
answer sentence selection and answer triggering.
"""

# TODO: Add a link to an official homepage for the dataset here
_HOMEPAGE = ""

# TODO: Add the licence for the dataset here if you can find it
_LICENSE = ""

# TODO: Add link to the official dataset URLs here
# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
types = {
    "answer_selection": "ass",
    "answer_triggering": "at",
}

modes = {"analysis": "json", "experiments": "tsv"}


class SelqaConfig(datasets.BuilderConfig):
    """ "BuilderConfig for SelQA Dataset"""

    def __init__(self, mode, type_, **kwargs):
        super(SelqaConfig, self).__init__(**kwargs)
        self.mode = mode
        self.type_ = type_


# TODO: Name of the dataset usually match the script name with CamelCase instead of snake_case
class Selqa(datasets.GeneratorBasedBuilder):
    """A New Benchmark for Selection-based Question Answering."""

    VERSION = datasets.Version("1.1.0")

    # This is an example of a dataset with multiple configurations.
    # If you don't want/need to define several sub-sets in your dataset,
    # just remove the BUILDER_CONFIG_CLASS and the BUILDER_CONFIGS attributes.

    # If you need to make complex sub-parts in the datasets with configurable options
    # You can create your own builder configuration class to store attribute, inheriting from datasets.BuilderConfig
    BUILDER_CONFIG_CLASS = SelqaConfig

    # You will be able to load one or the other configurations in the following list with
    # data = datasets.load_dataset('my_dataset', 'first_domain')
    # data = datasets.load_dataset('my_dataset', 'second_domain')
    BUILDER_CONFIGS = [
        SelqaConfig(
            name="answer_selection_analysis",
            mode="analysis",
            type_="answer_selection",
            version=VERSION,
            description="This part covers answer selection analysis",
        ),
        SelqaConfig(
            name="answer_selection_experiments",
            mode="experiments",
            type_="answer_selection",
            version=VERSION,
            description="This part covers answer selection experiments",
        ),
        SelqaConfig(
            name="answer_triggering_analysis",
            mode="analysis",
            type_="answer_triggering",
            version=VERSION,
            description="This part covers answer triggering analysis",
        ),
        SelqaConfig(
            name="answer_triggering_experiments",
            mode="experiments",
            type_="answer_triggering",
            version=VERSION,
            description="This part covers answer triggering experiments",
        ),
    ]

    DEFAULT_CONFIG_NAME = "answer_selection_analysis"  # It's not mandatory to have a default configuration. Just use one if it make sense.

    def _info(self):
        if (
            self.config.mode == "experiments"
        ):  # This is the name of the configuration selected in BUILDER_CONFIGS above
            features = datasets.Features(
                {
                    "question": datasets.Value("string"),
                    "candidate": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=["0", "1"]),
                }
            )
        else:
            if self.config.type_ == "answer_selection":
                features = datasets.Features(
                    {
                        "section": datasets.Value("string"),
                        "question": datasets.Value("string"),
                        "article": datasets.Value("string"),
                        "is_paraphrase": datasets.Value("bool"),
                        "topic": datasets.ClassLabel(
                            names=[
                                "MUSIC",
                                "TV",
                                "TRAVEL",
                                "ART",
                                "SPORT",
                                "COUNTRY",
                                "MOVIES",
                                "HISTORICAL EVENTS",
                                "SCIENCE",
                                "FOOD",
                            ]
                        ),
                        "answers": datasets.Sequence(datasets.Value("int32")),
                        "candidates": datasets.Sequence(datasets.Value("string")),
                        "q_types": datasets.Sequence(
                            datasets.ClassLabel(names=["what", "why", "when", "who", "where", "how", ""])
                        ),
                    }
                )
            else:
                features = datasets.Features(
                    {
                        "section": datasets.Value("string"),
                        "question": datasets.Value("string"),
                        "article": datasets.Value("string"),
                        "is_paraphrase": datasets.Value("bool"),
                        "topic": datasets.ClassLabel(
                            names=[
                                "MUSIC",
                                "TV",
                                "TRAVEL",
                                "ART",
                                "SPORT",
                                "COUNTRY",
                                "MOVIES",
                                "HISTORICAL EVENTS",
                                "SCIENCE",
                                "FOOD",
                            ]
                        ),
                        "q_types": datasets.Sequence(
                            datasets.ClassLabel(names=["what", "why", "when", "who", "where", "how", ""])
                        ),
                        "candidate_list": datasets.Sequence(
                            {
                                "article": datasets.Value("string"),
                                "section": datasets.Value("string"),
                                "candidates": datasets.Sequence(datasets.Value("string")),
                                "answers": datasets.Sequence(datasets.Value("int32")),
                            }
                        ),
                    }
                )
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO: This method is tasked with downloading/extracting the data and defining the splits depending on the configuration
        # If several configurations are possible (listed in BUILDER_CONFIGS), the configuration selected by the user is in self.config.name

        # dl_manager is a datasets.download.DownloadManager that can be used to download and extract URLs
        # It can accept any type or nested list/dict and will give back the same structure with the url replaced with path to local files.
        # By default the archives will be extracted and a path to a cached folder where they are extracted is returned instead of the archive
        urls = {
            "train": f"https://raw.githubusercontent.com/emorynlp/selqa/master/{types[self.config.type_]}/selqa-{types[self.config.type_]}-train.{modes[self.config.mode]}",
            "dev": f"https://raw.githubusercontent.com/emorynlp/selqa/master/{types[self.config.type_]}/selqa-{types[self.config.type_]}-dev.{modes[self.config.mode]}",
            "test": f"https://raw.githubusercontent.com/emorynlp/selqa/master/{types[self.config.type_]}/selqa-{types[self.config.type_]}-test.{modes[self.config.mode]}",
        }
        data_dir = dl_manager.download_and_extract(urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": data_dir["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": data_dir["test"], "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": data_dir["dev"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        # TODO: This method will receive as arguments the `gen_kwargs` defined in the previous `_split_generators` method.
        # It is in charge of opening the given file and yielding (key, example) tuples from the dataset
        # The key is not important, it's more here for legacy reason (legacy from tfds)
        with open(filepath, encoding="utf-8") as f:
            if self.config.mode == "experiments":
                csv_reader = csv.DictReader(
                    f, delimiter="\t", quoting=csv.QUOTE_NONE, fieldnames=["question", "candidate", "label"]
                )
                for id_, row in enumerate(csv_reader):
                    yield id_, row
            else:
                if self.config.type_ == "answer_selection":
                    for row in f:
                        data = json.loads(row)
                        for id_, item in enumerate(data):
                            yield id_, {
                                "section": item["section"],
                                "question": item["question"],
                                "article": item["article"],
                                "is_paraphrase": item["is_paraphrase"],
                                "topic": item["topic"],
                                "answers": item["answers"],
                                "candidates": item["candidates"],
                                "q_types": item["q_types"],
                            }
                else:
                    for row in f:
                        data = json.loads(row)
                        for id_, item in enumerate(data):
                            candidate_list = []
                            for entity in item["candidate_list"]:
                                candidate_list.append(
                                    {
                                        "article": entity["article"],
                                        "section": entity["section"],
                                        "answers": entity["answers"],
                                        "candidates": entity["candidates"],
                                    }
                                )
                            yield id_, {
                                "section": item["section"],
                                "question": item["question"],
                                "article": item["article"],
                                "is_paraphrase": item["is_paraphrase"],
                                "topic": item["topic"],
                                "q_types": item["q_types"],
                                "candidate_list": candidate_list,
                            }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Coached Conversational Preference Elicitation Dataset to Understanding Movie Preferences"""


import json
import os

import datasets


_CITATION = """\
@inproceedings{48414,
title	= {Coached Conversational Preference Elicitation: A Case Study in Understanding Movie Preferences},
author	= {Filip Radlinski and Krisztian Balog and Bill Byrne and Karthik Krishnamoorthi},
year	= {2019},
booktitle	= {Proceedings of the Annual SIGdial Meeting on Discourse and Dialogue}
}
"""

_DESCRIPTION = """\
A dataset consisting of 502 English dialogs with 12,000 annotated utterances between a user and an assistant discussing
movie preferences in natural language. It was collected using a Wizard-of-Oz methodology between two paid crowd-workers,
where one worker plays the role of an 'assistant', while the other plays the role of a 'user'. The 'assistant' elicits
the 'user’s' preferences about movies following a Coached Conversational Preference Elicitation (CCPE) method. The
assistant asks questions designed to minimize the bias in the terminology the 'user' employs to convey his or her
preferences as much as possible, and to obtain these preferences in natural language. Each dialog is annotated with
entity mentions, preferences expressed about entities, descriptions of entities provided, and other statements of
entities."""

_HOMEPAGE = "https://research.google/tools/datasets/coached-conversational-preference-elicitation/"

_LICENSE = "https://creativecommons.org/licenses/by-sa/4.0/"

_URLs = {"dataset": "https://storage.googleapis.com/dialog-data-corpus/CCPE-M-2019/data.json"}


class CoachedConvPrefConfig(datasets.BuilderConfig):
    """BuilderConfig for DialogRE"""

    def __init__(self, **kwargs):
        """BuilderConfig for DialogRE.
        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(CoachedConvPrefConfig, self).__init__(**kwargs)


class CoachedConvPref(datasets.GeneratorBasedBuilder):
    """Coached Conversational Preference Elicitation Dataset to Understanding Movie Preferences"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        CoachedConvPrefConfig(
            name="coached_conv_pref",
            version=datasets.Version("1.1.0"),
            description="Coached Conversational Preference Elicitation Dataset to Understanding Movie Preferences",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "conversationId": datasets.Value("string"),
                    "utterances": datasets.Sequence(
                        {
                            "index": datasets.Value("int32"),
                            "speaker": datasets.features.ClassLabel(names=["USER", "ASSISTANT"]),
                            "text": datasets.Value("string"),
                            "segments": datasets.Sequence(
                                {
                                    "startIndex": datasets.Value("int32"),
                                    "endIndex": datasets.Value("int32"),
                                    "text": datasets.Value("string"),
                                    "annotations": datasets.Sequence(
                                        {
                                            "annotationType": datasets.features.ClassLabel(
                                                names=[
                                                    "ENTITY_NAME",
                                                    "ENTITY_PREFERENCE",
                                                    "ENTITY_DESCRIPTION",
                                                    "ENTITY_OTHER",
                                                ]
                                            ),
                                            "entityType": datasets.features.ClassLabel(
                                                names=[
                                                    "MOVIE_GENRE_OR_CATEGORY",
                                                    "MOVIE_OR_SERIES",
                                                    "PERSON",
                                                    "SOMETHING_ELSE",
                                                ]
                                            ),
                                        }
                                    ),
                                }
                            ),
                        }
                    ),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        data_dir = dl_manager.download_and_extract(_URLs)

        # Dataset is a single corpus (does not contain any split)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir["dataset"]),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        # Empty Segment list with annotations dictionary
        # First prompt of a conversation does not contain the segment dictionary
        # We are setting it to None values
        segments_empty = [
            {
                "startIndex": 0,
                "endIndex": 0,
                "text": "",
                "annotations": [],
            }
        ]

        with open(filepath, encoding="utf-8") as f:
            dataset = json.load(f)

            for id_, data in enumerate(dataset):
                conversationId = data["conversationId"]

                utterances = data["utterances"]
                for utterance in utterances:
                    if "segments" not in utterance:
                        utterance["segments"] = segments_empty.copy()

                yield id_, {
                    "conversationId": conversationId,
                    "utterances": utterances,
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Wiki Asp datasert for Multi-domain Aspect-based Summarization"""


import json
import os

import datasets


# Find for instance the citation on arxiv or on the dataset repo/website
_CITATION = """\
@article{hayashi20tacl,
  title   = {WikiAsp: A Dataset for Multi-domain Aspect-based Summarization},
  authors = {Hiroaki Hayashi and Prashant Budania and Peng Wang and Chris Ackerson and Raj Neervannan and Graham Neubig},
  journal = {Transactions of the Association for Computational Linguistics (TACL)},
  year    = {2020},
  url     = {https://arxiv.org/abs/2011.07832}
}
"""

_DESCRIPTION = """\
WikiAsp is a multi-domain, aspect-based summarization dataset in the encyclopedic
domain. In this task, models are asked to summarize cited reference documents of a
Wikipedia article into aspect-based summaries. Each of the 20 domains include 10
domain-specific pre-defined aspects.
"""

_HOMEPAGE = "https://github.com/neulab/wikiasp"

_LICENSE = "CC BY-SA 4.0"

# Download links
_URLs = {
    "album": "http://phontron.com/download/wikiasp/Album.tar.bz2",
    "animal": "http://phontron.com/download/wikiasp/Animal.tar.bz2",
    "artist": "http://phontron.com/download/wikiasp/Artist.tar.bz2",
    "building": "http://phontron.com/download/wikiasp/Building.tar.bz2",
    "company": "http://phontron.com/download/wikiasp/Company.tar.bz2",
    "educational_institution": "http://phontron.com/download/wikiasp/EducationalInstitution.tar.bz2",
    "event": "http://phontron.com/download/wikiasp/Event.tar.bz2",
    "film": "http://phontron.com/download/wikiasp/Film.tar.bz2",
    "group": "http://phontron.com/download/wikiasp/Group.tar.bz2",
    "historic_place": "http://phontron.com/download/wikiasp/HistoricPlace.tar.bz2",
    "infrastructure": "http://phontron.com/download/wikiasp/Infrastructure.tar.bz2",
    "mean_of_transportation": "http://phontron.com/download/wikiasp/MeanOfTransportation.tar.bz2",
    "office_holder": "http://phontron.com/download/wikiasp/OfficeHolder.tar.bz2",
    "plant": "http://phontron.com/download/wikiasp/Plant.tar.bz2",
    "single": "http://phontron.com/download/wikiasp/Single.tar.bz2",
    "soccer_player": "http://phontron.com/download/wikiasp/SoccerPlayer.tar.bz2",
    "software": "http://phontron.com/download/wikiasp/Software.tar.bz2",
    "television_show": "http://phontron.com/download/wikiasp/TelevisionShow.tar.bz2",
    "town": "http://phontron.com/download/wikiasp/Town.tar.bz2",
    "written_work": "http://phontron.com/download/wikiasp/WrittenWork.tar.bz2",
}


class WikiAsp(datasets.GeneratorBasedBuilder):
    """TODO: Short description of my dataset."""

    VERSION = datasets.Version("1.1.0")

    # This is an example of a dataset with multiple configurations.
    # If you don't want/need to define several sub-sets in your dataset,
    # just remove the BUILDER_CONFIG_CLASS and the BUILDER_CONFIGS attributes.

    # If you need to make complex sub-parts in the datasets with configurable options
    # You can create your own builder configuration class to store attribute, inheriting from datasets.BuilderConfig
    # BUILDER_CONFIG_CLASS = MyBuilderConfig

    # You will  be able to load one or the other configurations in the following list with
    # data = datasets.load_dataset('my_dataset', 'first_domain')
    # data = datasets.load_dataset('my_dataset', 'second_domain')
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="album", version=VERSION, description="A subset of dataset from the musical album domain"
        ),
        datasets.BuilderConfig(
            name="animal", version=VERSION, description="A subset of dataset from the animal domain"
        ),
        datasets.BuilderConfig(
            name="artist", version=VERSION, description="A subset of dataset from the artist domain"
        ),
        datasets.BuilderConfig(
            name="building", version=VERSION, description="A subset of dataset from the buildings domain"
        ),
        datasets.BuilderConfig(
            name="company", version=VERSION, description="A subset of dataset from the company domain"
        ),
        datasets.BuilderConfig(
            name="educational_institution",
            version=VERSION,
            description="A subset of dataset from the educational institution domain",
        ),
        datasets.BuilderConfig(
            name="event", version=VERSION, description="A subset of dataset from the events domain"
        ),
        datasets.BuilderConfig(name="film", version=VERSION, description="A subset of dataset from the film domain"),
        datasets.BuilderConfig(name="group", version=VERSION, description="A subset of dataset from the group domain"),
        datasets.BuilderConfig(
            name="historic_place", version=VERSION, description="A subset of dataset from the historic places domain"
        ),
        datasets.BuilderConfig(
            name="infrastructure", version=VERSION, description="A subset of dataset from the infrastructure domain"
        ),
        datasets.BuilderConfig(
            name="mean_of_transportation",
            version=VERSION,
            description="A subset of dataset from the transportation mean domain",
        ),
        datasets.BuilderConfig(
            name="office_holder", version=VERSION, description="A subset of dataset from the office holder domain"
        ),
        datasets.BuilderConfig(name="plant", version=VERSION, description="A subset of dataset from the plant domain"),
        datasets.BuilderConfig(
            name="single", version=VERSION, description="A subset of dataset from the musical single domain"
        ),
        datasets.BuilderConfig(
            name="soccer_player", version=VERSION, description="A subset of dataset from the soccer player domain"
        ),
        datasets.BuilderConfig(
            name="software", version=VERSION, description="A subset of dataset from the software domain"
        ),
        datasets.BuilderConfig(
            name="television_show", version=VERSION, description="A subset of dataset from the television show domain"
        ),
        datasets.BuilderConfig(name="town", version=VERSION, description="A subset of dataset from the town domain"),
        datasets.BuilderConfig(
            name="written_work", version=VERSION, description="A subset of dataset from the written work domain"
        ),
    ]

    def _info(self):
        features = datasets.Features(
            {
                "exid": datasets.Value("string"),
                "inputs": datasets.Sequence(datasets.Value("string")),
                "targets": datasets.Sequence(datasets.Sequence(datasets.Value("string"))),
            }
        )
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        my_urls = _URLs[self.config.name]
        data_dir = dl_manager.download_and_extract(my_urls)
        data_dir = os.path.join(data_dir, self.config.name.title().replace("_", ""))
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "train.jsonl"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, "test.jsonl"), "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "valid.jsonl"),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                yield id_, {
                    "exid": data["exid"],
                    "inputs": data["inputs"],
                    "targets": data["targets"],
                }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Hyperpartisan News Detection"""


import os
import textwrap
import xml.etree.ElementTree as ET

import datasets


_CITATION = """\
@article{kiesel2019data,
  title={Data for pan at semeval 2019 task 4: Hyperpartisan news detection},
  author={Kiesel, Johannes and Mestre, Maria and Shukla, Rishabh and Vincent, Emmanuel and Corney, David and Adineh, Payam and Stein, Benno and Potthast, Martin},
  year={2019}
}
"""

_DESCRIPTION = """\
Hyperpartisan News Detection was a dataset created for PAN @ SemEval 2019 Task 4.
Given a news article text, decide whether it follows a hyperpartisan argumentation, i.e., whether it exhibits blind, prejudiced, or unreasoning allegiance to one party, faction, cause, or person.

There are 2 parts:
- byarticle: Labeled through crowdsourcing on an article basis. The data contains only articles for which a consensus among the crowdsourcing workers existed.
- bypublisher: Labeled by the overall bias of the publisher as provided by BuzzFeed journalists or MediaBiasFactCheck.com.
"""
_URL_BASE = "https://zenodo.org/record/1489920/files/"


class HyperpartisanNewsDetection(datasets.GeneratorBasedBuilder):
    """Hyperpartisan News Detection Dataset."""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="byarticle",
            version=datasets.Version("1.0.0", "Version Training and validation v1"),
            description=textwrap.dedent(
                """
                    This part of the data (filename contains "byarticle") is labeled through crowdsourcing on an article basis.
                    The data contains only articles for which a consensus among the crowdsourcing workers existed. It contains
                    a total of 645 articles. Of these, 238 (37%) are hyperpartisan and 407 (63%) are not, We will use a similar
                    (but balanced!) test set. Again, none of the publishers in this set will occur in the test set.
                """
            ),
        ),
        datasets.BuilderConfig(
            name="bypublisher",
            version=datasets.Version("1.0.0", "Version Training and validation v1"),
            description=textwrap.dedent(
                """
                    This part of the data (filename contains "bypublisher") is labeled by the overall bias of the publisher as provided
                    by BuzzFeed journalists or MediaBiasFactCheck.com. It contains a total of 750,000 articles, half of which (375,000)
                    are hyperpartisan and half of which are not. Half of the articles that are hyperpartisan (187,500) are on the left side
                    of the political spectrum, half are on the right side. This data is split into a training set (80%, 600,000 articles) and
                    a validation set (20%, 150,000 articles), where no publisher that occurs in the training set also occurs in the validation
                    set. Similarly, none of the publishers in those sets will occur in the test set.
                """
            ),
        ),
    ]

    def _info(self):
        features = {
            "text": datasets.Value("string"),
            "title": datasets.Value("string"),
            "hyperpartisan": datasets.Value("bool"),
            "url": datasets.Value("string"),
            "published_at": datasets.Value("string"),
        }

        if self.config.name == "bypublisher":
            # Bias is only included in the bypublisher config
            features["bias"] = datasets.ClassLabel(names=["right", "right-center", "least", "left-center", "left"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(features),
            supervised_keys=("text", "label"),
            homepage="https://pan.webis.de/semeval19/semeval19-web/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        urls = {
            datasets.Split.TRAIN: {
                "articles_file": _URL_BASE + "articles-training-" + self.config.name + "-20181122.zip?download=1",
                "labels_file": _URL_BASE + "ground-truth-training-" + self.config.name + "-20181122.zip?download=1",
            },
        }
        if self.config.name == "bypublisher":
            urls[datasets.Split.VALIDATION] = {
                "articles_file": _URL_BASE + "articles-training-" + self.config.name + "-20181122.zip?download=1",
                "labels_file": _URL_BASE + "ground-truth-training-" + self.config.name + "-20181122.zip?download=1",
            }

        data_dir = {}
        for key in urls:
            data_dir[key] = dl_manager.download_and_extract(urls[key])

        splits = []
        for split in data_dir:
            for key in data_dir[split]:
                data_dir[split][key] = os.path.join(data_dir[split][key], os.listdir(data_dir[split][key])[0])
            splits.append(datasets.SplitGenerator(name=split, gen_kwargs=data_dir[split]))
        return splits

    def _generate_examples(self, articles_file=None, labels_file=None):
        """Yields examples."""
        labels = {}
        with open(labels_file, "rb") as f_labels:
            tree = ET.parse(f_labels)
            root = tree.getroot()
            for label in root:
                article_id = label.attrib["id"]
                del label.attrib["labeled-by"]
                labels[article_id] = label.attrib

        with open(articles_file, "rb") as f_articles:
            tree = ET.parse(f_articles)
            root = tree.getroot()
            for idx, article in enumerate(root):
                example = {}
                example["title"] = article.attrib["title"]
                example["published_at"] = article.attrib.get("published-at", "")
                example["id"] = article.attrib["id"]
                example = {**example, **labels[example["id"]]}
                example["hyperpartisan"] = example["hyperpartisan"] == "true"

                example["text"] = ""
                for child in article:
                    example["text"] += ET.tostring(child).decode() + "\n"
                example["text"] = example["text"].strip()
                del example["id"]
                yield idx, example



import os

import datasets


_CITATION = """\
@article{cruz2020investigating,
  title={Investigating the True Performance of Transformers in Low-Resource Languages: A Case Study in Automatic Corpus Creation},
  author={Jan Christian Blaise Cruz and Jose Kristian Resabal and James Lin and Dan John Velasco and Charibeth Cheng},
  journal={arXiv preprint arXiv:2010.11574},
  year={2020}
}
"""

_DESCRIPTION = """\
Large-scale dataset of Filipino news articles. Sourced for the NewsPH-NLI Project (Cruz et al., 2020).
"""
_URL = "https://github.com/jcblaisecruz02/Filipino-Text-Benchmarks"
_LICENSE = "GPL-3.0"
_DATA_URL = "https://s3.us-east-2.amazonaws.com/blaisecruz.com/datasets/newsph"


class NewsphConfig(datasets.BuilderConfig):
    def __init__(self, data_url, **kwargs):
        super(NewsphConfig, self).__init__(
            version=datasets.Version(
                "1.0.0",
            ),
            **kwargs,
        )
        self.data_url = data_url


class Newsph(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        NewsphConfig(
            name="newsph",
            data_url=_DATA_URL + "/" + "newsph.zip",
            description=_DESCRIPTION,
        ),
    ]

    BUILDER_CONFIG_CLASS = NewsphConfig

    def _info(self):
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            features=datasets.Features({"text": datasets.Value("string")}),
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_URL,
            citation=_CITATION,
            license=_LICENSE,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_file = dl_manager.download_and_extract(self.config.data_url)
        data_dir = os.path.join(data_file, "newsph")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"data_file": os.path.join(data_dir, "train.txt"), "split": "train"},
            ),
        ]

    def _generate_examples(self, data_file, split):
        """Yields examples."""
        with open(data_file, encoding="utf-8") as f:
            for idx, row in enumerate(f):
                if row.strip():
                    yield idx, {"text": row}
                else:
                    yield idx, {"text": ""}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

import datasets


logger = datasets.logging.get_logger(__name__)

# Find for instance the citation on arxiv or on the dataset repo/website
_CITATION = """\
@article{dumitrescu2019introducing,
  title={Introducing RONEC--the Romanian Named Entity Corpus},
  author={Dumitrescu, Stefan Daniel and Avram, Andrei-Marius},
  journal={arXiv preprint arXiv:1909.01247},
  year={2019}
}
"""

# You can copy an official description
_DESCRIPTION = """\
RONEC - the Romanian Named Entity Corpus, at version 2.0, holds 12330 sentences with over 0.5M tokens, annotated with 15 classes, to a total of 80.283 distinctly annotated entities. It is used for named entity recognition and represents the largest Romanian NER corpus to date.
"""

_HOMEPAGE = "https://github.com/dumitrescustefan/ronec"

_LICENSE = "MIT License"

# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_URL = "https://raw.githubusercontent.com/dumitrescustefan/ronec/master/data/"
_TRAINING_FILE = "train.json"
_DEV_FILE = "valid.json"
_TEST_FILE = "test.json"


class RONECConfig(datasets.BuilderConfig):
    """BuilderConfig for RONEC dataset"""

    def __init__(self, **kwargs):
        super(RONECConfig, self).__init__(**kwargs)


class RONEC(datasets.GeneratorBasedBuilder):
    """RONEC dataset"""

    VERSION = datasets.Version("2.0.0")
    BUILDER_CONFIGS = [
        RONECConfig(name="ronec", version=VERSION, description="RONEC dataset"),
    ]

    def _info(self):
        features = datasets.Features(
            {
                "id": datasets.Value("int32"),
                "tokens": datasets.Sequence(datasets.Value("string")),
                "ner_ids": datasets.Sequence(datasets.Value("int32")),
                "space_after": datasets.Sequence(datasets.Value("bool")),
                "ner_tags": datasets.Sequence(
                    datasets.features.ClassLabel(
                        names=[
                            "O",
                            "B-PERSON",
                            "I-PERSON",
                            "B-ORG",
                            "I-ORG",
                            "B-GPE",
                            "I-GPE",
                            "B-LOC",
                            "I-LOC",
                            "B-NAT_REL_POL",
                            "I-NAT_REL_POL",
                            "B-EVENT",
                            "I-EVENT",
                            "B-LANGUAGE",
                            "I-LANGUAGE",
                            "B-WORK_OF_ART",
                            "I-WORK_OF_ART",
                            "B-DATETIME",
                            "I-DATETIME",
                            "B-PERIOD",
                            "I-PERIOD",
                            "B-MONEY",
                            "I-MONEY",
                            "B-QUANTITY",
                            "I-QUANTITY",
                            "B-NUMERIC",
                            "I-NUMERIC",
                            "B-ORDINAL",
                            "I-ORDINAL",
                            "B-FACILITY",
                            "I-FACILITY",
                        ]
                    )
                ),
            }
        )

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        urls_to_download = {"train": _URL + _TRAINING_FILE, "dev": _URL + _DEV_FILE, "test": _URL + _TEST_FILE}

        downloaded_files = dl_manager.download(urls_to_download)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": downloaded_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": downloaded_files["dev"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": downloaded_files["test"]},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""

        logger.info("⏳ Generating examples from = %s", filepath)
        with open(filepath, "r", encoding="utf-8") as f:
            data = json.load(f)
            for instance in data:
                yield instance["id"], instance



import json
import os

import datasets


_CITATION = """\
@article{scialom2020mlsum,
  title={MLSUM: The Multilingual Summarization Corpus},
  author={Scialom, Thomas and Dray, Paul-Alexis and Lamprier, Sylvain and Piwowarski, Benjamin and Staiano, Jacopo},
  journal={arXiv preprint arXiv:2004.14900},
  year={2020}
}
"""

_DESCRIPTION = """\
We present MLSUM, the first large-scale MultiLingual SUMmarization dataset.
Obtained from online newspapers, it contains 1.5M+ article/summary pairs in five different languages -- namely, French, German, Spanish, Russian, Turkish.
Together with English newspapers from the popular CNN/Daily mail dataset, the collected data form a large scale multilingual dataset which can enable new research directions for the text summarization community.
We report cross-lingual comparative analyses based on state-of-the-art systems.
These highlight existing biases which motivate the use of a multi-lingual dataset.
"""
_URL = "https://gitlab.lip6.fr/scialom/mlsum_data/-/raw/master/MLSUM/"
_LANG = ["de", "es", "fr", "ru", "tu"]


class Mlsum(datasets.GeneratorBasedBuilder):

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name=lang,
            version=datasets.Version("1.0.0"),
            description="",
        )
        for lang in _LANG
    ]

    def _info(self):
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "summary": datasets.Value("string"),
                    "topic": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "date": datasets.Value("string")
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs

        lang = str(self.config.name)
        urls_to_download = {
            "test": _URL + lang + "_test.zip",
            "train": _URL + lang + "_train.zip",
            "validation": _URL + lang + "_val.zip",
        }
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(downloaded_files["train"], lang + "_train.jsonl"),
                    "lang": lang,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(downloaded_files["validation"], lang + "_val.jsonl"),
                    "lang": lang,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(downloaded_files["test"], lang + "_test.jsonl"),
                    "lang": lang,
                },
            ),
        ]

    def _generate_examples(self, filepath, lang):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            i = 0
            for line in f:
                data = json.loads(line)
                i += 1
                yield i, {
                    "text": data["text"],
                    "summary": data["summary"],
                    "topic": data["topic"],
                    "url": data["url"],
                    "title": data["title"],
                    "date": data["date"],
                }



import json
import os
import random

import datasets


random.seed(42)  # This is important, to ensure the same order for concept sets as the official script.

_CITATION = """\
@inproceedings{lin-etal-2020-commongen,
    title = "{C}ommon{G}en: A Constrained Text Generation Challenge for Generative Commonsense Reasoning",
    author = "Lin, Bill Yuchen  and
      Zhou, Wangchunshu  and
      Shen, Ming  and
      Zhou, Pei  and
      Bhagavatula, Chandra  and
      Choi, Yejin  and
      Ren, Xiang",
    booktitle = "Findings of the Association for Computational Linguistics: EMNLP 2020",
    month = nov,
    year = "2020",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/2020.findings-emnlp.165",
    doi = "10.18653/v1/2020.findings-emnlp.165",
    pages = "1823--1840"
}
"""

_DESCRIPTION = """\
CommonGen is a constrained text generation task, associated with a benchmark dataset,
to explicitly test machines for the ability of generative commonsense reasoning. Given
a set of common concepts; the task is to generate a coherent sentence describing an
everyday scenario using these concepts.

CommonGen is challenging because it inherently requires 1) relational reasoning using
background commonsense knowledge, and 2) compositional generalization ability to work
on unseen concept combinations. Our dataset, constructed through a combination of
crowd-sourcing from AMT and existing caption corpora, consists of 30k concept-sets and
50k sentences in total.
"""
_URL = "https://storage.googleapis.com/huggingface-nlp/datasets/common_gen/commongen_data.zip"


class CommonGen(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("2020.5.30")

    def _info(self):
        features = datasets.Features(
            {
                "concept_set_idx": datasets.Value("int32"),
                "concepts": datasets.Sequence(datasets.Value("string")),
                "target": datasets.Value("string"),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=datasets.info.SupervisedKeysData(input="concepts", output="target"),
            homepage="https://inklab.usc.edu/CommonGen/index.html",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        dl_dir = dl_manager.download_and_extract(_URL)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(dl_dir, "commongen.train.jsonl"), "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": os.path.join(dl_dir, "commongen.dev.jsonl"), "split": "dev"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(dl_dir, "commongen.test_noref.jsonl"), "split": "test"},
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            id_ = 0
            for idx, row in enumerate(f):
                row = row.replace(", }", "}")  # Fix possible JSON format error
                data = json.loads(row)

                rand_order = [word for word in data["concept_set"].split("#")]
                random.shuffle(rand_order)

                if split == "test":
                    yield idx, {
                        "concept_set_idx": idx,
                        "concepts": rand_order,
                        "target": "",
                    }
                else:
                    for scene in data["scene"]:
                        yield id_, {
                            "concept_set_idx": idx,
                            "concepts": rand_order,
                            "target": scene,
                        }
                        id_ += 1



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Hate speech dataset"""


import csv
import os

import datasets


_CITATION = """\
@inproceedings{gibert2018hate,
    title = "{Hate Speech Dataset from a White Supremacy Forum}",
    author = "de Gibert, Ona  and
      Perez, Naiara  and
      Garcia-Pablos, Aitor  and
      Cuadros, Montse",
    booktitle = "Proceedings of the 2nd Workshop on Abusive Language Online ({ALW}2)",
    month = oct,
    year = "2018",
    address = "Brussels, Belgium",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/W18-5102",
    doi = "10.18653/v1/W18-5102",
    pages = "11--20",
}
"""

_DESCRIPTION = """\
These files contain text extracted from Stormfront, a white supremacist forum. A random set of
forums posts have been sampled from several subforums and split into sentences. Those sentences
have been manually labelled as containing hate speech or not, according to certain annotation guidelines.
"""

_DATA_URL = "https://github.com/Vicomtech/hate-speech-dataset/archive/master.zip"


class HateSpeech18(datasets.GeneratorBasedBuilder):
    """Hate speech dataset"""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "user_id": datasets.Value("int64"),
                    "subforum_id": datasets.Value("int64"),
                    "num_contexts": datasets.Value("int64"),
                    "label": datasets.features.ClassLabel(names=["noHate", "hate", "idk/skip", "relation"]),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/Vicomtech/hate-speech-dataset",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        dl_dir = dl_manager.download_and_extract(_DATA_URL)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"filepath": os.path.join(dl_dir, "hate-speech-dataset-master")}
            ),
        ]

    def _generate_examples(self, filepath):

        with open(os.path.join(filepath, "annotations_metadata.csv"), encoding="utf-8") as csv_file:

            csv_reader = csv.reader(
                csv_file, quotechar='"', delimiter=",", quoting=csv.QUOTE_ALL, skipinitialspace=True
            )

            next(csv_reader)

            for idx, row in enumerate(csv_reader):

                file_id, user_id, subforum_id, num_contexts, label = row

                all_files_path = os.path.join(filepath, "all_files")

                path = os.path.join(all_files_path, file_id + ".txt")

                with open(path, encoding="utf-8") as file:
                    text = file.read()

                yield idx, {
                    "text": text,
                    "user_id": user_id,
                    "subforum_id": subforum_id,
                    "num_contexts": num_contexts,
                    "label": label,
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""SentiWS: German-language resource for sentiment analysis, pos-tagging"""


import os

import datasets


_CITATION = """\
@INPROCEEDINGS{remquahey2010,
title = {SentiWS -- a Publicly Available German-language Resource for Sentiment Analysis},
booktitle = {Proceedings of the 7th International Language Resources and Evaluation (LREC'10)},
author = {Remus, R. and Quasthoff, U. and Heyer, G.},
year = {2010}
}
"""

_DESCRIPTION = """\
SentimentWortschatz, or SentiWS for short, is a publicly available German-language resource for sentiment analysis, and pos-tagging. The POS tags are ["NN", "VVINF", "ADJX", "ADV"] -> ["noun", "verb", "adjective", "adverb"], and positive and negative polarity bearing words are weighted within the interval of [-1, 1].
"""

_HOMEPAGE = ""

_LICENSE = "Creative Commons Attribution-Noncommercial-Share Alike 3.0 Unported License"

_URLs = ["https://pcai056.informatik.uni-leipzig.de/downloads/etc/SentiWS/SentiWS_v2.0.zip"]


class SentiWS(datasets.GeneratorBasedBuilder):
    """SentiWS: German-language resource for sentiment analysis, pos-tagging"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="pos-tagging", version=VERSION, description="This covers pos-tagging task"),
        datasets.BuilderConfig(
            name="sentiment-scoring",
            version=VERSION,
            description="This covers the sentiment-scoring in [-1, 1] corresponding to (negative, positive) sentiment",
        ),
    ]

    DEFAULT_CONFIG_NAME = "pos-tagging"

    def _info(self):

        if (
            self.config.name == "pos-tagging"
        ):  # the pos-tags are ["NN", "VVINF", "ADJX", "ADV"] -> ["noun", "verb", "adjective", "adverb"]
            features = datasets.Features(
                {
                    "word": datasets.Value("string"),
                    "pos-tag": datasets.ClassLabel(names=["NN", "VVINF", "ADJX", "ADV"]),
                }
            )
        else:  # This is an example to show how to have different features for "first_domain" and "second_domain"
            features = datasets.Features(
                {
                    "word": datasets.Value("string"),
                    "sentiment-score": datasets.Value("float32"),
                }
            )
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO: This method is tasked with downloading/extracting the data and defining the splits depending on the configuration
        # If several configurations are possible (listed in BUILDER_CONFIGS), the configuration selected by the user is in self.config.name

        # dl_manager is a datasets.download.DownloadManager that can be used to download and extract URLs
        # It can accept any type or nested list/dict and will give back the same structure with the url replaced with path to local files.
        # By default the archives will be extracted and a path to a cached folder where they are extracted is returned instead of the archive
        my_urls = _URLs
        data_dir = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "sourcefiles": [
                        os.path.join(data_dir[0], f)
                        for f in ["SentiWS_v2.0_Positive.txt", "SentiWS_v2.0_Negative.txt"]
                    ],
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, sourcefiles, split):
        """Yields examples."""
        # TODO: This method will receive as arguments the `gen_kwargs` defined in the previous `_split_generators` method.
        # It is in charge of opening the given file and yielding (key, example) tuples from the dataset
        # The key is not important, it's more here for legacy reason (legacy from tfds)
        for file_idx, filepath in enumerate(sourcefiles):
            with open(filepath, encoding="utf-8") as f:
                for id_, row in enumerate(f):
                    word = row.split("|")[0]
                    if self.config.name == "pos-tagging":
                        tag = row.split("|")[1].split("\t")[0]
                        yield f"{file_idx}_{id_}", {"word": word, "pos-tag": tag}
                    else:
                        sentiscore = row.split("|")[1].split("\t")[1]
                        yield f"{file_idx}_{id_}", {"word": word, "sentiment-score": float(sentiscore)}



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
import datasets


_DESCRIPTION = """\
GermaNER is a freely available statistical German Named Entity Tagger based on conditional random fields(CRF). The tagger is trained and evaluated on the NoSta-D Named Entity dataset, which was used in the GermEval 2014 for named entity recognition. The tagger comes close to the performance of the best (proprietary) system in the competition with 77% F-measure (this is the latest result; the one reported in the paper is 76%) test set performance on the four standard NER classes (PERson, LOCation, ORGanisation and OTHer).

We describe a range of features and their influence on German NER classification and provide a comparative evaluation and some analysis of the results. The software components, the training data and all data used for feature generation are distributed under permissive licenses, thus this tagger can be used in academic and commercial settings without restrictions or fees. The tagger is available as a command-line tool and as an Apache UIMA component.
"""
_HOMEPAGE_URL = "https://github.com/tudarmstadt-lt/GermaNER"
_URL = "https://raw.githubusercontent.com/tudarmstadt-lt/GermaNER/a206b554feca263d740302449fff0776c66d0040/data/v0.9.1/full_train.tsv"
_CITATION = """\
@inproceedings{Benikova2015GermaNERFO,
  title={GermaNER: Free Open German Named Entity Recognition Tool},
  author={Darina Benikova and S. Yimam and Prabhakaran Santhanam and Chris Biemann},
  booktitle={GSCL},
  year={2015}
}
"""


class GermaNER(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("0.9.1")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "B-LOC",
                                "B-ORG",
                                "B-OTH",
                                "B-PER",
                                "I-LOC",
                                "I-ORG",
                                "I-OTH",
                                "I-PER",
                                "O",
                            ]
                        )
                    ),
                },
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        path = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"datapath": path},
            )
        ]

    def _generate_examples(self, datapath):
        sentence_counter = 0
        with open(datapath, encoding="utf-8") as f:
            current_words = []
            current_labels = []
            for row in f:
                row = row.rstrip()
                row_split = row.split()
                if len(row_split) == 2:
                    token, label = row_split
                    current_words.append(token)
                    current_labels.append(label)
                else:
                    if not current_words:
                        continue
                    assert len(current_words) == len(current_labels), "word len doesnt match label length"
                    sentence = (
                        sentence_counter,
                        {
                            "id": str(sentence_counter),
                            "tokens": current_words,
                            "ner_tags": current_labels,
                        },
                    )
                    sentence_counter += 1
                    current_words = []
                    current_labels = []
                    yield sentence

            # if something remains:
            if current_words:
                sentence = (
                    sentence_counter,
                    {
                        "id": str(sentence_counter),
                        "tokens": current_words,
                        "ner_tags": current_labels,
                    },
                )
                yield sentence



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""WMT MLQE Shared task 3."""


import csv
import os

import datasets


_CITATION = """
Not available.
"""

_DESCRIPTION = """\
This shared task (part of WMT20) will build on its previous editions
to further examine automatic methods for estimating the quality
of neural machine translation output at run-time, without relying
on reference translations. As in previous years, we cover estimation
at various levels. Important elements introduced this year include: a new
task where sentences are annotated with Direct Assessment (DA)
scores instead of labels based on post-editing; a new multilingual
sentence-level dataset mainly from Wikipedia articles, where the
source articles can be retrieved for document-wide context; the
availability of NMT models to explore system-internal information for the task.

The goal of this task 3 is to predict document-level quality scores as well as fine-grained annotations.
"""

_HOMEPAGE = "http://www.statmt.org/wmt20/quality-estimation-task.html"

_LICENSE = "Unknown"

_URLs = {
    "train+dev": "https://github.com/deep-spin/deep-spin.github.io/raw/master/docs/data/wmt2020_qe/qe-task3-enfr-traindev.tar.gz",
    "test": "https://github.com/deep-spin/deep-spin.github.io/raw/master/docs/data/wmt2020_qe/qe-enfr-blindtest.tar.gz",
}


_ANNOTATION_CATEGORIES = [
    "Addition",
    "Agreement",
    "Ambiguous Translation",
    "Capitalization",
    "Character Encoding",
    "Company Terminology",
    "Date/Time",
    "Diacritics",
    "Duplication",
    "False Friend",
    "Grammatical Register",
    "Hyphenation",
    "Inconsistency",
    "Lexical Register",
    "Lexical Selection",
    "Named Entity",
    "Number",
    "Omitted Auxiliary Verb",
    "Omitted Conjunction",
    "Omitted Determiner",
    "Omitted Preposition",
    "Omitted Pronoun",
    "Orthography",
    "Other POS Omitted",
    "Over-translation",
    "Overly Literal",
    "POS",
    "Punctuation",
    "Shouldn't Have Been Translated",
    "Shouldn't have been translated",
    "Spelling",
    "Tense/Mood/Aspect",
    "Under-translation",
    "Unidiomatic",
    "Unintelligible",
    "Unit Conversion",
    "Untranslated",
    "Whitespace",
    "Word Order",
    "Wrong Auxiliary Verb",
    "Wrong Conjunction",
    "Wrong Determiner",
    "Wrong Language Variety",
    "Wrong Preposition",
    "Wrong Pronoun",
]


class Wmt20MlqeTask3(datasets.GeneratorBasedBuilder):
    """WMT MLQE Shared task 3."""

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="plain_text",
            version=datasets.Version("1.1.0"),
            description="Plain text",
        )
    ]

    def _info(self):
        features = datasets.Features(
            {
                "document_id": datasets.Value("string"),
                "source_segments": datasets.Sequence(datasets.Value("string")),
                "source_tokenized": datasets.Sequence(datasets.Value("string")),
                "mt_segments": datasets.Sequence(datasets.Value("string")),
                "mt_tokenized": datasets.Sequence(datasets.Value("string")),
                "annotations": datasets.Sequence(
                    {
                        "segment_id": datasets.Sequence(datasets.Value("int32")),
                        "annotation_start": datasets.Sequence(datasets.Value("int32")),
                        "annotation_length": datasets.Sequence(datasets.Value("int32")),
                        "severity": datasets.ClassLabel(names=["minor", "major", "critical"]),
                        "severity_weight": datasets.Value("float32"),
                        "category": datasets.ClassLabel(names=_ANNOTATION_CATEGORIES),
                    }
                ),
                "token_annotations": datasets.Sequence(
                    {
                        "segment_id": datasets.Sequence(datasets.Value("int32")),
                        "first_token": datasets.Sequence(datasets.Value("int32")),
                        "last_token": datasets.Sequence(datasets.Value("int32")),
                        "token_after_gap": datasets.Sequence(datasets.Value("int32")),
                        "severity": datasets.ClassLabel(names=["minor", "major", "critical"]),
                        "category": datasets.ClassLabel(names=_ANNOTATION_CATEGORIES),
                    }
                ),
                "token_index": datasets.Sequence(datasets.Sequence(datasets.Sequence(datasets.Value("int32")))),
                "total_words": datasets.Value("int32"),
            }
        )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        downloaded_files = dl_manager.download(_URLs)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "main_dir": "task3/train",
                    "split": "train",
                    "files": dl_manager.iter_archive(downloaded_files["train+dev"]),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "main_dir": "test-blind",
                    "split": "test",
                    "files": dl_manager.iter_archive(downloaded_files["test"]),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "main_dir": "task3/dev",
                    "split": "dev",
                    "files": dl_manager.iter_archive(downloaded_files["train+dev"]),
                },
            ),
        ]

    def _generate_examples(self, main_dir, split, files):
        """Yields examples."""

        prev_folder = None
        source_segments, source_tokenized, mt_segments, mt_tokenized = [None] * 4
        token_index, total_words, annotations, token_annotations = [], [], [], []
        for path, f in files:
            if path.startswith(main_dir):
                dir_name = path.split("/")[main_dir.count("/") + 1]
                folder = main_dir + "/" + dir_name

                if prev_folder is not None and prev_folder != folder:
                    yield prev_folder, {
                        "document_id": os.path.basename(prev_folder),
                        "source_segments": source_segments,
                        "source_tokenized": source_tokenized,
                        "mt_segments": mt_segments,
                        "mt_tokenized": mt_tokenized,
                        "annotations": annotations,
                        "token_annotations": token_annotations,
                        "token_index": token_index,
                        "total_words": total_words,
                    }
                    source_segments, source_tokenized, mt_segments, mt_tokenized = [None] * 4
                    token_index, total_words, annotations, token_annotations = [], [], [], []

                prev_folder = folder

                source_segments_path = "/".join([folder, "source.segments"])
                source_tokenized_path = "/".join([folder, "source.tokenized"])
                mt_segments_path = "/".join([folder, "mt.segments"])
                mt_tokenized_path = "/".join([folder, "mt.tokenized"])
                total_words_path = "/".join([folder, "total_words"])
                token_index_path = "/".join([folder, "token_index"])

                if path == source_segments_path:
                    source_segments = f.read().decode("utf-8").splitlines()
                elif path == source_tokenized_path:
                    source_tokenized = f.read().decode("utf-8").splitlines()
                elif path == mt_segments_path:
                    mt_segments = f.read().decode("utf-8").splitlines()
                elif path == mt_tokenized_path:
                    mt_tokenized = f.read().decode("utf-8").splitlines()
                elif path == total_words_path:
                    total_words = f.read().decode("utf-8").splitlines()[0]
                elif path == token_index_path:
                    token_index = [
                        [idx.split(" ") for idx in line.split("\t")]
                        for line in f.read().decode("utf-8").splitlines()
                        if line != ""
                    ]

                if split in ["train", "dev"]:
                    annotations_path = "/".join([folder, "annotations.tsv"])
                    token_annotations_path = "/".join([folder, "token_annotations.tsv"])

                    if path == annotations_path:
                        lines = (line.decode("utf-8") for line in f)
                        reader = csv.DictReader(lines, delimiter="\t")
                        annotations = [
                            {
                                "segment_id": row["segment_id"].split(" "),
                                "annotation_start": row["annotation_start"].split(" "),
                                "annotation_length": row["annotation_length"].split(" "),
                                "severity": row["severity"],
                                "severity_weight": row["severity_weight"],
                                "category": row["category"],
                            }
                            for row in reader
                        ]
                    elif path == token_annotations_path:
                        lines = (line.decode("utf-8") for line in f)
                        reader = csv.DictReader(lines, delimiter="\t")
                        token_annotations = [
                            {
                                "segment_id": row["segment_id"].split(" "),
                                "first_token": row["first_token"].replace("-", "-1").split(" "),
                                "last_token": row["last_token"].replace("-", "-1").split(" "),
                                "token_after_gap": row["token_after_gap"].replace("-", "-1").split(" "),
                                "severity": row["severity"],
                                "category": row["category"],
                            }
                            for row in reader
                        ]
        if prev_folder is not None:
            yield prev_folder, {
                "document_id": os.path.basename(prev_folder),
                "source_segments": source_segments,
                "source_tokenized": source_tokenized,
                "mt_segments": mt_segments,
                "mt_tokenized": mt_tokenized,
                "annotations": annotations,
                "token_annotations": token_annotations,
                "token_index": token_index,
                "total_words": total_words,
            }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Euroconst: A parallel corpus collected from the European Constitution for 21 language"""


import itertools
import os

import datasets


_CITATION = """\
J. Tiedemann, 2012, Parallel Data, Tools and Interfaces in OPUS. \
In Proceedings of the 8th International Conference on Language Resources and Evaluation (LREC 2012)
"""


_DESCRIPTION = """\
A parallel corpus collected from the European Constitution for 21 language.
"""


_HOMEPAGE = "http://opus.nlpl.eu/EUconst.php"

_LANGUAGES = [
    "cs",
    "da",
    "de",
    "el",
    "en",
    "es",
    "et",
    "fi",
    "fr",
    "ga",
    "hu",
    "it",
    "lt",
    "lv",
    "mt",
    "nl",
    "pl",
    "pt",
    "sk",
    "sl",
    "sv",
]
_LANGUAGE_PAIRS = list(itertools.combinations(_LANGUAGES, 2))

_BASE_URL = "http://opus.nlpl.eu/download.php?f=EUconst/v1/moses"
_URLS = {f"{l1}-{l2}": f"{_BASE_URL}/{l1}-{l2}.txt.zip" for l1, l2 in _LANGUAGE_PAIRS}


class OpusEuconst(datasets.GeneratorBasedBuilder):
    """Euroconst: A parallel corpus collected from the European Constitution for 21 language"""

    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name=f"{l1}-{l2}", version=datasets.Version("1.0.0"), description=f"OPUS EUconst {l1}-{l2}"
        )
        for l1, l2 in _LANGUAGE_PAIRS
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {"translation": datasets.features.Translation(languages=tuple(self.config.name.split("-")))}
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        lang_pair = self.config.name.split("-")
        data_dir = dl_manager.download_and_extract(_URLS[self.config.name])
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "source_file": os.path.join(data_dir, f"EUconst.{self.config.name}.{lang_pair[0]}"),
                    "target_file": os.path.join(data_dir, f"EUconst.{self.config.name}.{lang_pair[1]}"),
                },
            ),
        ]

    def _generate_examples(self, source_file, target_file):
        with open(source_file, encoding="utf-8") as f:
            source_sentences = f.read().split("\n")
        with open(target_file, encoding="utf-8") as f:
            target_sentences = f.read().split("\n")

        assert len(target_sentences) == len(source_sentences), "Sizes do not match: %d vs %d for %s vs %s." % (
            len(source_sentences),
            len(target_sentences),
            source_file,
            target_file,
        )

        source, target = tuple(self.config.name.split("-"))
        for idx, (l1, l2) in enumerate(zip(source_sentences, target_sentences)):
            result = {"translation": {source: l1, target: l2}}
            yield idx, result



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""KorQuAD v1.0:The Korean Question Answering Dataset"""


import json

import datasets
from datasets.tasks import QuestionAnsweringExtractive


_CITATION = """\
@article{lim2019korquad1,
  title={Korquad1. 0: Korean qa dataset for machine reading comprehension},
  author={Lim, Seungyoung and Kim, Myungji and Lee, Jooyoul},
  journal={arXiv preprint arXiv:1909.07005},
  year={2019}
}
"""

_DESCRIPTION = """\
KorQuAD 1.0 is a large-scale Korean dataset for machine reading comprehension task consisting of human generated questions for Wikipedia articles. We benchmark the data collecting process of SQuADv1.0 and crowdsourced 70,000+ question-answer pairs. 1,637 articles and 70,079 pairs of question answers were collected. 1,420 articles are used for the training set, 140 for the dev set, and 77 for the test set. 60,407 question-answer pairs are for the training set, 5,774 for the dev set, and 3,898 for the test set.
"""
_HOMEPAGE = "https://korquad.github.io/KorQuad%201.0/"
_LICENSE = "CC BY-ND 2.0 KR"

_URL = "https://korquad.github.io/dataset/"
_URLS = {
    "train": _URL + "KorQuAD_v1.0_train.json",
    "dev": _URL + "KorQuAD_v1.0_dev.json",
}


class SquadKorV1(datasets.GeneratorBasedBuilder):
    """KorQuAD 1.0 dataset"""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="squad_kor_v1",
            version=VERSION,
            description=_DESCRIPTION,
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "context": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answers": datasets.features.Sequence(
                        {
                            "text": datasets.Value("string"),
                            "answer_start": datasets.Value("int32"),
                        }
                    ),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[
                QuestionAnsweringExtractive(
                    question_column="question", context_column="context", answers_column="answers"
                )
            ],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # download and extract URLs
        urls_to_download = _URLS
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["dev"]}),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            squad = json.load(f)
            for example in squad["data"]:
                title = example.get("title", "").strip()
                for paragraph in example["paragraphs"]:
                    context = paragraph["context"].strip()
                    for qa in paragraph["qas"]:
                        question = qa["question"].strip()
                        id_ = qa["id"]

                        answer_starts = [answer["answer_start"] for answer in qa["answers"]]
                        answers = [answer["text"].strip() for answer in qa["answers"]]

                        yield id_, {
                            "title": title,
                            "context": context,
                            "question": question,
                            "id": id_,
                            "answers": {
                                "answer_start": answer_starts,
                                "text": answers,
                            },
                        }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
import json
import os

import datasets


_DESCRIPTION = """\
ToTTo is an open-domain English table-to-text dataset with over 120,000 training examples that proposes a controlled generation task: given a Wikipedia table and a set of highlighted table cells, produce a one-sentence description.
"""
_HOMEPAGE_URL = ""
_URL = "https://storage.googleapis.com/totto-public/totto_data.zip"
_CITATION = """\
@inproceedings{parikh2020totto,
  title={{ToTTo}: A Controlled Table-To-Text Generation Dataset},
  author={Parikh, Ankur P and Wang, Xuezhi and Gehrmann, Sebastian and Faruqui, Manaal and Dhingra, Bhuwan and Yang, Diyi and Das, Dipanjan},
  booktitle={Proceedings of EMNLP},
  year={2020}
 }
"""


class Totto(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.0.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "table_page_title": datasets.Value("string"),
                    "table_webpage_url": datasets.Value("string"),
                    "table_section_title": datasets.Value("string"),
                    "table_section_text": datasets.Value("string"),
                    "table": [
                        [
                            {
                                "column_span": datasets.Value("int32"),
                                "is_header": datasets.Value("bool"),
                                "row_span": datasets.Value("int32"),
                                "value": datasets.Value("string"),
                            }
                        ]
                    ],
                    "highlighted_cells": datasets.Sequence(datasets.Sequence(datasets.Value("int32"))),
                    "example_id": datasets.Value("string"),
                    "sentence_annotations": datasets.Sequence(
                        {
                            "original_sentence": datasets.Value("string"),
                            "sentence_after_deletion": datasets.Value("string"),
                            "sentence_after_ambiguity": datasets.Value("string"),
                            "final_sentence": datasets.Value("string"),
                        }
                    ),
                    "overlap_subset": datasets.Value("string"),
                },
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        path = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "datapath": os.path.join(path, "totto_data/totto_train_data.jsonl"),
                    "datatype": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "datapath": os.path.join(path, "totto_data/totto_dev_data.jsonl"),
                    "datatype": "valid",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "datapath": os.path.join(path, "totto_data/unlabeled_totto_test_data.jsonl"),
                    "datatype": "test",
                },
            ),
        ]

    def _generate_examples(self, datapath, datatype):
        with open(datapath, "r", encoding="utf-8") as json_file:
            json_list = list(json_file)

        for example_counter, json_str in enumerate(json_list):
            result = json.loads(json_str)
            response = {
                "id": example_counter,
                "table_page_title": result["table_page_title"],
                "table_webpage_url": result["table_webpage_url"],
                "table_section_title": result["table_section_title"],
                "table_section_text": result["table_section_text"],
                "table": result["table"],
                "highlighted_cells": result["highlighted_cells"],
                "example_id": str(result["example_id"]),
            }
            if datatype == "train":
                response["overlap_subset"] = "none"
            else:
                response["overlap_subset"] = str(result["overlap_subset"])

            if datatype == "test":
                response["sentence_annotations"] = [
                    {
                        "original_sentence": "none",
                        "sentence_after_deletion": "none",
                        "sentence_after_ambiguity": "none",
                        "final_sentence": "none",
                    }
                ]
            else:
                response["sentence_annotations"] = result["sentence_annotations"]
            yield example_counter, response



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""DailyDialog: A Manually Labelled Multi-turn Dialogue Dataset"""


import os
from zipfile import ZipFile

import datasets


_CITATION = """\
@InProceedings{li2017dailydialog,
    author = {Li, Yanran and Su, Hui and Shen, Xiaoyu and Li, Wenjie and Cao, Ziqiang and Niu, Shuzi},
    title = {DailyDialog: A Manually Labelled Multi-turn Dialogue Dataset},
    booktitle = {Proceedings of The 8th International Joint Conference on Natural Language Processing (IJCNLP 2017)},
    year = {2017}
}
"""

_DESCRIPTION = """\
We develop a high-quality multi-turn dialog dataset, DailyDialog, which is intriguing in several aspects.
The language is human-written and less noisy. The dialogues in the dataset reflect our daily communication way
and cover various topics about our daily life. We also manually label the developed dataset with communication
intention and emotion information. Then, we evaluate existing approaches on DailyDialog dataset and hope it
benefit the research field of dialog systems.
"""

_URL = "http://yanran.li/files/ijcnlp_dailydialog.zip"

act_label = {
    "0": "__dummy__",  # Added to be compatible out-of-the-box with datasets.ClassLabel
    "1": "inform",
    "2": "question",
    "3": "directive",
    "4": "commissive",
}

emotion_label = {
    "0": "no emotion",
    "1": "anger",
    "2": "disgust",
    "3": "fear",
    "4": "happiness",
    "5": "sadness",
    "6": "surprise",
}


class DailyDialog(datasets.GeneratorBasedBuilder):
    """DailyDialog: A Manually Labelled Multi-turn Dialogue Dataset"""

    VERSION = datasets.Version("1.0.0")

    __EOU__ = "__eou__"

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "dialog": datasets.features.Sequence(datasets.Value("string")),
                    "act": datasets.features.Sequence(datasets.ClassLabel(names=list(act_label.values()))),
                    "emotion": datasets.features.Sequence(datasets.ClassLabel(names=list(emotion_label.values()))),
                }
            ),
            supervised_keys=None,
            homepage="http://yanran.li/dailydialog",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager):
        """Returns SplitGenerators."""
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)
        data_dir = os.path.join(dl_dir, "ijcnlp_dailydialog")

        # The splits are nested inside the zip
        for name in ("train", "validation", "test"):
            zip_fpath = os.path.join(data_dir, f"{name}.zip")
            with ZipFile(zip_fpath) as zip_file:
                zip_file.extractall(path=data_dir)
                zip_file.close()

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "file_path": os.path.join(data_dir, "train", "dialogues_train.txt"),
                    "act_path": os.path.join(data_dir, "train", "dialogues_act_train.txt"),
                    "emotion_path": os.path.join(data_dir, "train", "dialogues_emotion_train.txt"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "file_path": os.path.join(data_dir, "test", "dialogues_test.txt"),
                    "act_path": os.path.join(data_dir, "test", "dialogues_act_test.txt"),
                    "emotion_path": os.path.join(data_dir, "test", "dialogues_emotion_test.txt"),
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "file_path": os.path.join(data_dir, "validation", "dialogues_validation.txt"),
                    "act_path": os.path.join(data_dir, "validation", "dialogues_act_validation.txt"),
                    "emotion_path": os.path.join(data_dir, "validation", "dialogues_emotion_validation.txt"),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, file_path, act_path, emotion_path, split):
        """Yields examples."""
        # Yields (key, example) tuples from the dataset
        with open(file_path, "r", encoding="utf-8") as f, open(act_path, "r", encoding="utf-8") as act, open(
            emotion_path, "r", encoding="utf-8"
        ) as emotion:
            for i, (line_f, line_act, line_emotion) in enumerate(zip(f, act, emotion)):
                if len(line_f.strip()) == 0:
                    break
                dialog = line_f.split(self.__EOU__)[:-1]
                act = line_act.split(" ")[:-1]
                emotion = line_emotion.split(" ")[:-1]

                assert len(dialog) == len(act) == len(emotion), "Different turns btw dialogue & emotion & action"

                yield f"{split}-{i}", {
                    "dialog": dialog,
                    "act": [act_label[x] for x in act],
                    "emotion": [emotion_label[x] for x in emotion],
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Dengue Dataset Low-Resource Multiclass Text Classification Dataset in Filipino"""

import csv
import os

import datasets


_DESCRIPTION = """\
    Benchmark dataset for low-resource multiclass classification, with 4,015 training, 500 testing, and 500 validation examples, each labeled as part of five classes. Each sample can be a part of multiple classes. Collected as tweets.
"""

_CITATION = """\
    @INPROCEEDINGS{8459963,
      author={E. D. {Livelo} and C. {Cheng}},
      booktitle={2018 IEEE International Conference on Agents (ICA)},
      title={Intelligent Dengue Infoveillance Using Gated Recurrent Neural Learning and Cross-Label Frequencies},
      year={2018},
      volume={},
      number={},
      pages={2-7},
      doi={10.1109/AGENTS.2018.8459963}}
    }
"""

_HOMEPAGE = "https://github.com/jcblaisecruz02/Filipino-Text-Benchmarks"

# TODO: Add the licence for the dataset here if you can find it
_LICENSE = ""

_URL = "https://s3.us-east-2.amazonaws.com/blaisecruz.com/datasets/dengue/dengue_raw.zip"


class DengueFilipino(datasets.GeneratorBasedBuilder):
    """Dengue Dataset Low-Resource Multiclass Text Classification Dataset in Filipino"""

    VERSION = datasets.Version("1.0.0")

    def _info(self):
        features = datasets.Features(
            {
                "text": datasets.Value("string"),
                "absent": datasets.features.ClassLabel(names=["0", "1"]),
                "dengue": datasets.features.ClassLabel(names=["0", "1"]),
                "health": datasets.features.ClassLabel(names=["0", "1"]),
                "mosquito": datasets.features.ClassLabel(names=["0", "1"]),
                "sick": datasets.features.ClassLabel(names=["0", "1"]),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URL)
        train_path = os.path.join(data_dir, "dengue", "train.csv")
        test_path = os.path.join(data_dir, "dengue", "train.csv")
        validation_path = os.path.join(data_dir, "dengue", "valid.csv")

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_path,
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": validation_path,
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(
                csv_file, quotechar='"', delimiter=",", quoting=csv.QUOTE_ALL, skipinitialspace=True
            )
            next(csv_reader)
            for id_, row in enumerate(csv_reader):
                try:
                    text, absent, dengue, health, mosquito, sick = row
                    payload = {
                        "text": text,
                        "absent": absent,
                        "dengue": dengue,
                        "health": health,
                        "mosquito": mosquito,
                        "sick": sick,
                    }
                    yield id_, payload
                except ValueError:
                    pass



"""TODO(squad_it): Add a description here."""


import json

import datasets
from datasets.tasks import QuestionAnsweringExtractive


# TODO(squad_it): BibTeX citation
_CITATION = """\
@InProceedings{10.1007/978-3-030-03840-3_29,
    author={Croce, Danilo and Zelenanska, Alexandra and Basili, Roberto},
    editor={Ghidini, Chiara and Magnini, Bernardo and Passerini, Andrea and Traverso, Paolo",
    title={Neural Learning for Question Answering in Italian},
    booktitle={AI*IA 2018 -- Advances in Artificial Intelligence},
    year={2018},
    publisher={Springer International Publishing},
    address={Cham},
    pages={389--402},
    isbn={978-3-030-03840-3}
}
"""

# TODO(squad_it):
_DESCRIPTION = """\
SQuAD-it is derived from the SQuAD dataset and it is obtained through semi-automatic translation of the SQuAD dataset
into Italian. It represents a large-scale dataset for open question answering processes on factoid questions in Italian.
 The dataset contains more than 60,000 question/answer pairs derived from the original English dataset. The dataset is
 split into training and test sets to support the replicability of the benchmarking of QA systems:
"""

_URL = "https://github.com/crux82/squad-it/raw/master/"
_URLS = {
    "train": _URL + "SQuAD_it-train.json.gz",
    "test": _URL + "SQuAD_it-test.json.gz",
}


class SquadIt(datasets.GeneratorBasedBuilder):
    """TODO(squad_it): Short description of my dataset."""

    # TODO(squad_it): Set up version.
    VERSION = datasets.Version("0.1.0")

    def _info(self):
        # TODO(squad_it): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "context": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answers": datasets.features.Sequence(
                        {
                            "text": datasets.Value("string"),
                            "answer_start": datasets.Value("int32"),
                        }
                    ),
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/crux82/squad-it",
            citation=_CITATION,
            task_templates=[
                QuestionAnsweringExtractive(
                    question_column="question", context_column="context", answers_column="answers"
                )
            ],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(squad_it): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        urls_to_download = _URLS
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": downloaded_files["test"]}),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(squad_it): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for example in data["data"]:
                for paragraph in example["paragraphs"]:
                    context = paragraph["context"].strip()
                    for qa in paragraph["qas"]:
                        question = qa["question"].strip()
                        id_ = qa["id"]

                        answer_starts = [answer["answer_start"] for answer in qa["answers"]]
                        answers = [answer["text"].strip() for answer in qa["answers"]]

                        yield id_, {
                            "context": context,
                            "question": question,
                            "id": id_,
                            "answers": {
                                "answer_start": answer_starts,
                                "text": answers,
                            },
                        }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""GAP is a gender-balanced text data set."""


import csv

import datasets


_CITATION = """
@article{DBLP:journals/corr/abs-1810-05201,
  author    = {Kellie Webster and
               Marta Recasens and
               Vera Axelrod and
               Jason Baldridge},
  title     = {Mind the {GAP:} {A} Balanced Corpus of Gendered Ambiguous Pronouns},
  journal   = {CoRR},
  volume    = {abs/1810.05201},
  year      = {2018},
  url       = {http://arxiv.org/abs/1810.05201},
  archivePrefix = {arXiv},
  eprint    = {1810.05201},
  timestamp = {Tue, 30 Oct 2018 20:39:56 +0100},
  biburl    = {https://dblp.org/rec/bib/journals/corr/abs-1810-05201},
  bibsource = {dblp computer science bibliography, https://dblp.org}
}
"""

_DESCRIPTION = """
GAP is a gender-balanced dataset containing 8,908 coreference-labeled pairs of
(ambiguous pronoun, antecedent name), sampled from Wikipedia and released by
Google AI Language for the evaluation of coreference resolution in practical
applications.
"""

_TRAINURL = "https://raw.githubusercontent.com/google-research-datasets/gap-coreference/master/gap-development.tsv"
_VALIDATIONURL = "https://raw.githubusercontent.com/google-research-datasets/gap-coreference/master/gap-validation.tsv"
_TESTURL = "https://raw.githubusercontent.com/google-research-datasets/gap-coreference/master/gap-test.tsv"


class Gap(datasets.GeneratorBasedBuilder):
    """GAP is a gender-balanced dataset.

    It contains 8,908 coreference-labeled pairs
    of (ambiguous pronoun, antecedent name), sampled from Wikipedia.
    """

    VERSION = datasets.Version("0.1.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "ID": datasets.Value("string"),
                    "Text": datasets.Value("string"),
                    "Pronoun": datasets.Value("string"),
                    "Pronoun-offset": datasets.Value("int32"),
                    "A": datasets.Value("string"),
                    "A-offset": datasets.Value("int32"),
                    "A-coref": datasets.Value("bool"),
                    "B": datasets.Value("string"),
                    "B-offset": datasets.Value("int32"),
                    "B-coref": datasets.Value("bool"),
                    "URL": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/google-research-datasets/gap-coreference",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        directory = dl_manager.download_and_extract(
            {"train": _TRAINURL, "validation": _VALIDATIONURL, "test": _TESTURL}
        )
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": directory["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": directory["validation"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": directory["test"]},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as tsvfile:
            reader = csv.DictReader(tsvfile, dialect="excel-tab")
            for i, row in enumerate(reader):
                row["A-coref"] = row["A-coref"] == "TRUE"
                row["B-coref"] = row["B-coref"] == "TRUE"
                row["A-offset"] = int(row["A-offset"])
                row["B-offset"] = int(row["B-offset"])
                row["Pronoun-offset"] = int(row["Pronoun-offset"])
                yield i, row



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TODO: Add a description here."""


import os
from collections import namedtuple

import datasets


_CITATION = """\
@inproceedings{Toutanova2016ADA,
  title={A Dataset and Evaluation Metrics for Abstractive Compression of Sentences and Short Paragraphs},
  author={Kristina Toutanova and Chris Brockett and Ke M. Tran and Saleema Amershi},
  booktitle={EMNLP},
  year={2016}
}
"""

_DESCRIPTION = """\
This dataset contains sentences and short paragraphs with corresponding shorter (compressed) versions. There are up to five compressions for each input text, together with quality judgements of their meaning preservation and grammaticality. The dataset is derived using source texts from the Open American National Corpus (ww.anc.org) and crowd-sourcing.
"""

_HOMEPAGE = "https://msropendata.com/datasets/f8ce2ec9-7fbd-48f7-a8bb-2d2279373563"

_LICENSE = "Microsoft Research Data License Agreement"


_SOURCE_LABELS = ["source_id", "domain", "source_text"]
_COMPRESSION_LABELS = ["compressed_text", "judge_id", "num_ratings", "ratings"]
SourceInfo = namedtuple("SourceInfo", _SOURCE_LABELS)
CompressionInfo = namedtuple("CompressionInfo", _COMPRESSION_LABELS)


class MsrTextCompression(datasets.GeneratorBasedBuilder):
    """This dataset contains sentences and short paragraphs with corresponding shorter (compressed) versions. There are up to five compressions for each input text, together with quality judgements of their meaning preservation and grammaticality. The dataset is derived using source texts from the Open American National Corpus (ww.anc.org) and crowd-sourcing."""

    VERSION = datasets.Version("1.1.0")
    _ENCODING = "utf-8-sig"

    @property
    def manual_download_instructions(self):
        return """\
  You should download the dataset from https://www.microsoft.com/en-us/download/details.aspx?id=54262
  The webpage requires registration.
  Unzip and please put the files from the extracted RawData folder under the following names
  train.tsv, valid.tsv, and test.tsv in a dir of your choice,
  which will be used as a manual_dir, e.g. `~/.manual_dir/msr_text_compression`
  The data can then be loaded via:
  `datasets.load_dataset("msr_text_compression", data_dir="~/.manual_dir/msr_text_compression")`.
  """

    def _info(self):

        # Define features
        source = {k: datasets.Value("string") for k in _SOURCE_LABELS}
        target = {
            "compressed_text": datasets.Value("string"),
            "judge_id": datasets.Value("string"),
            "num_ratings": datasets.Value("int64"),
            "ratings": datasets.Sequence(datasets.Value("int64")),
        }
        targets = {"targets": datasets.Sequence(target)}
        feature_dict = {**source, **targets}

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(feature_dict),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = os.path.abspath(os.path.expanduser(dl_manager.manual_dir))
        if not os.path.exists(data_dir):
            raise FileNotFoundError(
                f"{data_dir} does not exist. Make sure you insert a manual dir via `datasets.load_dataset('msr_text_compression', data_dir=...)` per the manual download instructions: {self.manual_download_instructions}"
            )
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"input_file": os.path.join(data_dir, "train.tsv")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"input_file": os.path.join(data_dir, "valid.tsv")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"input_file": os.path.join(data_dir, "test.tsv")},
            ),
        ]

    def _parse_source(self, s):
        source_id, domain, text = [x.strip() for x in s.split("\t")]
        return SourceInfo(source_id, domain, text)._asdict()

    def _parse_ratings(self, num_ratings, ratings):
        """Parses raw ratings into list of ints
        Args:
            num_ratings: int
            ratings: List[str]
        Returns:
            List[int] with len == num_ratings
        """

        # ratings contains both numeric ratings (actual ratings) and qualitative descriptions
        # we only wish to keep the numeric ratings
        assert num_ratings * 2 == len(ratings)

        return [int(r) for r in ratings[:: len(ratings) // num_ratings]]

    def _parse_target(self, target):
        text, judge, num_ratings, *ratings = [t.strip() for t in target.split("\t")]
        num_ratings = int(num_ratings)
        ratings = self._parse_ratings(num_ratings, ratings)
        return CompressionInfo(text, judge, num_ratings, ratings)._asdict()

    def _generate_examples(self, input_file):
        """Yields examples.

        Files are encoded with BOM markers, hence the use of utf-8-sig as codec
        """
        with open(input_file, encoding=self._ENCODING) as f:
            for id_, line in enumerate(f):
                source_info, *targets_info = line.split("|||")

                source = self._parse_source(source_info)
                targets = {"targets": [self._parse_target(target) for target in targets_info]}

                yield id_, {**source, **targets}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Dataset built from <auto-generated, manually corrected> caption pairs of
YouTube videos with labels capturing the differences between the two."""


import json

import datasets


_CITATION = ""

_DESCRIPTION = """\
Dataset built from pairs of YouTube captions where both 'auto-generated' and
'manually-corrected' captions are available for a single specified language.
This dataset labels two-way (e.g. ignoring single-sided insertions) same-length
token differences in the `diff_type` column. The `default_seq` is composed of
tokens from the 'auto-generated' captions. When a difference occurs between
the 'auto-generated' vs 'manually-corrected' captions types, the `correction_seq`
contains tokens from the 'manually-corrected' captions.
"""

_LICENSE = "MIT License"

_RELEASE_TAG = "v1.0"
_NUM_FILES = 4
_URLS = [
    f"https://raw.githubusercontent.com/2dot71mily/youtube_captions_corrections/{_RELEASE_TAG}/data/transcripts/en/split/youtube_caption_corrections_{i}.json"
    for i in range(_NUM_FILES)
]


class YoutubeCaptionCorrections(datasets.GeneratorBasedBuilder):
    """YouTube captions corrections."""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "video_ids": datasets.Value("string"),
                    "default_seq": datasets.Sequence(datasets.Value("string")),
                    "correction_seq": datasets.Sequence(datasets.Value("string")),
                    "diff_type": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "NO_DIFF",
                                "CASE_DIFF",
                                "PUNCUATION_DIFF",
                                "CASE_AND_PUNCUATION_DIFF",
                                "STEM_BASED_DIFF",
                                "DIGIT_DIFF",
                                "INTRAWORD_PUNC_DIFF",
                                "UNKNOWN_TYPE_DIFF",
                                "RESERVED_DIFF",
                            ]
                        )
                    ),
                }
            ),
            supervised_keys=("correction_seq", "diff_type"),
            homepage="https://github.com/2dot71mily/youtube_captions_corrections",
            license=_LICENSE,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        downloaded_filepaths = dl_manager.download_and_extract(_URLS)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepaths": downloaded_filepaths},
            ),
        ]

    def _generate_examples(self, filepaths):
        """Yields examples."""
        for file_idx, fp in enumerate(filepaths):
            with open(fp, "r", encoding="utf-8") as json_file:
                json_lists = list(json_file)
            for line_idx, json_list_str in enumerate(json_lists):
                json_list = json.loads(json_list_str)

                for ctr_idx, result in enumerate(json_list):
                    response = {
                        "video_ids": result["video_ids"],
                        "diff_type": result["diff_type"],
                        "default_seq": result["default_seq"],
                        "correction_seq": result["correction_seq"],
                    }
                    yield f"{file_idx}_{line_idx}_{ctr_idx}", response



"""XOR QA: Cross-lingual Open-Retrieval Question Answering"""


import json
import textwrap

import datasets


_XOR_TYDI_QA_CITATION = """\
    @misc{asai2020xor,
      title={XOR QA: Cross-lingual Open-Retrieval Question Answering},
      author={Akari Asai and Jungo Kasai and Jonathan H. Clark and Kenton Lee and Eunsol Choi and Hannaneh Hajishirzi},
      year={2020},
      eprint={2010.11856},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_XOR_TYDI_QA_DESCRIPTION = """\
    XOR-TyDi QA brings together for the first time information-seeking questions,
    open-retrieval QA, and multilingual QA to create a multilingual open-retrieval
    QA dataset that enables cross-lingual answer retrieval. It consists of questions
    written by information-seeking native speakers in 7 typologically diverse languages
    and answer annotations that are retrieved from multilingual document collections.
    There are three sub-tasks: XOR-Retrieve, XOR-EnglishSpan, and XOR-Full.
"""

_DESCRIPTIONS = {
    "xor-retrieve": textwrap.dedent(
        """\
        XOR-Retrieve is a cross-lingual retrieval task where a question is written in the target
        language (e.g., Japanese) and a system is required to retrieve English document that answers the question.
        """
    ),
    "xor-full": textwrap.dedent(
        """\
        XOR-Full is a cross-lingual retrieval task where a question is written in the target
        language (e.g., Japanese) and a system is required to output a short answer in the target language."""
    ),
}

_DATA_URLS = {
    "xor-retrieve": {
        "train": "https://nlp.cs.washington.edu/xorqa/XORQA_site/data/xor_train_retrieve_eng_span.jsonl",
        "dev": "https://nlp.cs.washington.edu/xorqa/XORQA_site/data/xor_dev_retrieve_eng_span_v1_1.jsonl",
        "test": "https://nlp.cs.washington.edu/xorqa/XORQA_site/data/xor_test_retrieve_eng_span_q_only_v1_1.jsonl",
    },
    "xor-full": {
        "train": "https://nlp.cs.washington.edu/xorqa/XORQA_site/data/xor_train_full.jsonl",
        "dev": "https://nlp.cs.washington.edu/xorqa/XORQA_site/data/xor_dev_full_v1_1.jsonl",
        "test": "https://nlp.cs.washington.edu/xorqa/XORQA_site/data/xor_test_full_q_only_v1_1.jsonl",
    },
}

_XOR_TYDI_QA_URL = "https://nlp.cs.washington.edu/xorqa/"


class XORTyDiConfig(datasets.BuilderConfig):
    "BuilderConfig for XOR-TyDi Dataset"

    def __init__(self, data_url, citation, url, **kwargs):
        """
        Args:

        data_url: `dictionary`, dict with url for each split of data.
        citation: `string`, citation for the dataset.
        url: `string`, url for information about the dataset.
        **kwargs: keyword arguments forwarded to super.
        """
        super(XORTyDiConfig, self).__init__(version=datasets.Version("1.1.0", ""), **kwargs)
        self.data_url = data_url
        self.citation = citation
        self.url = url


class XORTyDi(datasets.GeneratorBasedBuilder):

    BUILDER_CONFIGS = [
        XORTyDiConfig(
            name=name,
            description=_DESCRIPTIONS[name],
            data_url=_DATA_URLS[name],
            citation=_XOR_TYDI_QA_CITATION,
            url=_XOR_TYDI_QA_URL,
        )
        for name in ["xor-retrieve", "xor-full"]
    ]

    def _info(self):
        features = {}
        features["question"] = datasets.Value("string")
        features["lang"] = datasets.features.ClassLabel(names=["ar", "bn", "fi", "ja", "ko", "ru", "te"])
        features["answers"] = datasets.Value("string")

        return datasets.DatasetInfo(
            description=_XOR_TYDI_QA_DESCRIPTION + "\n" + self.config.description,
            features=datasets.Features(features),
            homepage=self.config.url,
            citation=_XOR_TYDI_QA_CITATION,
        )

    def _split_generators(self, dl_manager):
        train = dl_manager.download_and_extract(self.config.data_url["train"])
        dev = dl_manager.download_and_extract(self.config.data_url["dev"])
        test = dl_manager.download_and_extract(self.config.data_url["test"])

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train, "split": "train"}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": dev, "split": "dev"}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": test, "split": "test"}),
        ]

    def _generate_examples(self, filepath, split):
        with open(filepath, encoding="utf-8") as f:
            jlines = f.read()
            result = [json.loads(jline) for jline in jlines.splitlines()]
            if split == "test":
                for id_, row in enumerate(result):
                    yield id_, {"question": row["question"], "answers": "None", "lang": row["lang"].strip()}
            else:
                for id_, row in enumerate(result):
                    yield id_, {
                        "question": row["question"],
                        "answers": " ".join(row["answers"]),
                        "lang": row["lang"].strip(),
                    }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# flake8: noqa
# Lint as: python3
"""Util import."""

from . import logging
from .download_manager import DownloadManager, GenerateMode
from .file_utils import DownloadConfig, cached_path, hf_bucket_url, is_remote_url, relative_to_absolute_path, temp_seed
from .mock_download_manager import MockDownloadManager
from .py_utils import (
    NonMutableDict,
    classproperty,
    copyfunc,
    dumps,
    first_non_null_value,
    flatten_nest_dict,
    has_sufficient_disk_space,
    map_nested,
    memoize,
    no_op_if_value_is_null,
    size_str,
    temporary_assignment,
    unique_values,
    zip_dict,
    zip_nested,
)
from .tqdm_utils import disable_progress_bar, is_progress_bar_enabled, set_progress_bar_enabled, tqdm
from .version import Version



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Commonsense Explanations (CoS-E) Dataset."""


import json

import datasets


_CITATION = """
@inproceedings{rajani2019explain,
     title = {Explain Yourself! Leveraging Language models for Commonsense Reasoning},
    author = {Rajani, Nazneen Fatema  and
      McCann, Bryan  and
      Xiong, Caiming  and
      Socher, Richard}
      year={2019}
    booktitle = {Proceedings of the 2019 Conference of the Association for Computational Linguistics (ACL2019)}
    url ={https://arxiv.org/abs/1906.02361}
}
"""

_DESCRIPTION = """
Common Sense Explanations (CoS-E) allows for training language models to
automatically generate explanations that can be used during training and
inference in a novel Commonsense Auto-Generated Explanation (CAGE) framework.
"""

_COS_E_URL = "https://raw.githubusercontent.com/salesforce/cos-e/master/data/"

# COS E has explanations for the CQA dataset, which is joined by ID.
_CQA_V1_11_URL_TRAIN = "https://s3.amazonaws.com/commensenseqa/train_rand_split.jsonl"
_CQA_V1_11_URL_DEV = "https://s3.amazonaws.com/commensenseqa/dev_rand_split.jsonl"
_CQA_V1_11_URL_TEST = "https://s3.amazonaws.com/commensenseqa/test_rand_split_no_answers.jsonl"

_CQA_V1_0_URL_TRAIN = _COS_E_URL + "v1.0/train_rand_split.jsonl"
_CQA_V1_0_URL_DEV = _COS_E_URL + "v1.0/dev_rand_split.jsonl"
_CQA_V1_0_URL_TEST = _COS_E_URL + "v1.0/test_rand_split_no_answers.jsonl"


def _download_and_index_cqa(dl_manager, name):
    """Downloads CQA and returns it, indexed by id, for joining with Cos-E."""

    downloaded_files = dl_manager.download_and_extract(
        {
            "cqa_train": _CQA_V1_11_URL_TRAIN if name == "v1.11" else _CQA_V1_0_URL_TRAIN,
            "cqa_dev": _CQA_V1_11_URL_DEV if name == "v1.11" else _CQA_V1_0_URL_DEV,
            "cqa_test": _CQA_V1_11_URL_TEST if name == "v1.11" else _CQA_V1_0_URL_TEST,
        }
    )

    # NB: "cqa_test" is included in the files, but not in any of the CoS-E splits.
    cqa_splits = ["cqa_train", "cqa_dev"]
    cqa_complete = []
    for split in cqa_splits:
        with open(downloaded_files[split], encoding="utf-8") as f:
            for _, line in enumerate(f):
                d = json.loads(line)
                cqa_complete.append(d)

        # Index the CQA dataset by id for joining with Cos-E.
        cqa_indexed = {}
    for d in cqa_complete:
        cqa_indexed[d["id"]] = d
    return cqa_indexed


def _get_choices_and_answer(cqa):
    """Returns choices and the answer from a cqa example."""
    choices = []
    answer_key = cqa["answerKey"]
    answer = None
    for choice in cqa["question"]["choices"]:
        choices.append(choice["text"])
        if answer_key == choice["label"]:
            answer = choice["text"]
    return choices, answer


class CosEConfig(datasets.BuilderConfig):

    """BuilderConfig for CosE"""

    def __init__(self, **kwargs):
        """

        Args:
            **kwargs: keyword arguments forwarded to super.
        """
        super(CosEConfig, self).__init__(**kwargs)


class CosE(datasets.GeneratorBasedBuilder):
    """CoS-E: Common Sense Explanations corpus."""

    BUILDER_CONFIGS = [
        CosEConfig(
            name="v1.0",
            description="cos-e version 1.0",
            version=datasets.Version("1.0.0", ""),
        ),
        CosEConfig(
            name="v1.11",
            description="cos-e version 1.11",
            version=datasets.Version("1.11.0", ""),
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "choices": datasets.features.Sequence(datasets.Value("string")),
                    "answer": datasets.Value("string"),
                    "abstractive_explanation": datasets.Value("string"),
                    "extractive_explanation": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/salesforce/cos-e",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        # NB: The CQA Dataset should be read only once, and only by callers who
        # want to _create_ the Cos-E dataset from scratch.
        cqa_indexed = _download_and_index_cqa(dl_manager, self.config.name)

        if self.config.name == "v1.11":
            files = dl_manager.download_and_extract(
                {
                    "dev": [_COS_E_URL + "v1.11/cose_dev_v1.11_processed.jsonl"],
                    "train": [_COS_E_URL + "v1.11/cose_train_v1.11_processed.jsonl"],
                }
            )

        elif self.config.name == "v1.0":
            files = dl_manager.download_and_extract(
                {
                    "dev": [_COS_E_URL + "v1.0/cose_dev_v1.0_processed.jsonl"],
                    "train": [_COS_E_URL + "v1.0/cose_train_v1.0_processed.jsonl"],
                }
            )
        else:
            raise ValueError("Unknown config name")
        # We use the CoS-E/CQA dev set as our validation set.
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"files": files["train"], "cqa_indexed": cqa_indexed},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"files": files["dev"], "cqa_indexed": cqa_indexed},
            ),
        ]

    def _generate_examples(self, files, **kwargs):
        """Yields examples."""
        cqa_indexed = kwargs["cqa_indexed"]
        for filepath in files:
            with open(filepath, encoding="utf-8") as f:
                for line in f:
                    cos = json.loads(line)
                    cqa = cqa_indexed[cos["id"]]
                    choices, answer = _get_choices_and_answer(cqa)
                    yield cos["id"], {
                        "id": cos["id"],
                        "question": cqa["question"]["stem"],
                        "choices": choices,
                        "answer": answer,
                        "abstractive_explanation": cos["explanation"]["open-ended"],
                        "extractive_explanation": cos["explanation"]["selected"],
                    }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""NELL: Never Ending Language Learner"""


from html import unescape
from urllib.parse import unquote

import datasets


_CITATION = """\
@inproceedings{mitchell2015,
  added-at = {2015-01-27T15:35:24.000+0100},
  author = {Mitchell, T. and Cohen, W. and Hruscha, E. and Talukdar, P. and Betteridge, J. and Carlson, A. and Dalvi, B. and Gardner, M. and Kisiel, B. and Krishnamurthy, J. and Lao, N. and Mazaitis, K. and Mohammad, T. and Nakashole, N. and Platanios, E. and Ritter, A. and Samadi, M. and Settles, B. and Wang, R. and Wijaya, D. and Gupta, A. and Chen, X. and Saparov, A. and Greaves, M. and Welling, J.},
  biburl = {https://www.bibsonomy.org/bibtex/263070703e6bb812852cca56574aed093/hotho},
  booktitle = {AAAI},
  description = {Papers by William W. Cohen},
  interhash = {52d0d71f6f5b332dabc1412f18e3a93d},
  intrahash = {63070703e6bb812852cca56574aed093},
  keywords = {learning nell ontology semantic toread},
  note = {: Never-Ending Learning in AAAI-2015},
  timestamp = {2015-01-27T15:35:24.000+0100},
  title = {Never-Ending Learning},
  url = {http://www.cs.cmu.edu/~wcohen/pubs.html},
  year = 2015
}
"""


_DESCRIPTION = """This dataset provides version 1115 of the belief
extracted by CMU's Never Ending Language Learner (NELL) and version
1110 of the candidate belief extracted by NELL. See
http://rtw.ml.cmu.edu/rtw/overview.  NELL is an open information
extraction system that attempts to read the Clueweb09 of 500 million
web pages (http://boston.lti.cs.cmu.edu/Data/clueweb09/) and general
web searches.

The dataset has 4 configurations: nell_belief, nell_candidate,
nell_belief_sentences, and nell_candidate_sentences. nell_belief is
certainties of belief are lower. The two sentences config extracts the
CPL sentence patterns filled with the applicable 'best' literal string
for the entities filled into the sentence patterns. And also provides
sentences found using web searches containing the entities and
relationships.

There are roughly 21M entries for nell_belief_sentences, and 100M
sentences for nell_candidate_sentences.
"""


_LICENSE = """
"""

_URLs = {
    "nell_belief": "http://rtw.ml.cmu.edu/resources/results/08m/NELL.08m.1115.esv.csv.gz",
    "nell_candidate": "http://rtw.ml.cmu.edu/resources/results/08m/NELL.08m.1110.cesv.csv.gz",
    "nell_belief_sentences": "http://rtw.ml.cmu.edu/resources/results/08m/NELL.08m.1115.esv.csv.gz",
    "nell_candidate_sentences": "http://rtw.ml.cmu.edu/resources/results/08m/NELL.08m.1110.cesv.csv.gz",
}


class Nell(datasets.GeneratorBasedBuilder):
    """NELL dataset for knowledge bases and knowledge graphs and underlying sentences."""

    VERSION = datasets.Version("0.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="nell_belief", description="The beliefs in raw data form", version="1115.0.0"),
        datasets.BuilderConfig(
            name="nell_candidate", description="The candidate beliefs in raw data form", version="1110.0.0"
        ),
        datasets.BuilderConfig(
            name="nell_belief_sentences",
            description="The underlying sentences available for the nell beliefs",
            version="1115.0.0",
        ),
        datasets.BuilderConfig(
            name="nell_candidate_sentences",
            description="The underlying sentences available for the nell candidate beliefs",
            version="1110.0.0",
        ),
    ]

    DEFAULT_CONFIG_NAME = "nell"

    def _info(self):
        if self.config.name in ("nell_belief", "nell_candidate"):
            features = datasets.Features(
                {
                    "entity": datasets.Value("string"),
                    "relation": datasets.Value("string"),
                    "value": datasets.Value("string"),
                    "iteration_of_promotion": datasets.Value("string"),
                    "score": datasets.Value("string"),
                    "source": datasets.Value("string"),
                    "entity_literal_strings": datasets.Value("string"),
                    "value_literal_strings": datasets.Value("string"),
                    "best_entity_literal_string": datasets.Value("string"),
                    "best_value_literal_string": datasets.Value("string"),
                    "categories_for_entity": datasets.Value("string"),
                    "categories_for_value": datasets.Value("string"),
                    "candidate_source": datasets.Value("string"),
                }
            )
        else:
            features = datasets.Features(
                {
                    "entity": datasets.Value("string"),
                    "relation": datasets.Value("string"),
                    "value": datasets.Value("string"),
                    "score": datasets.Value("string"),
                    "sentence": datasets.Value("string"),
                    "count": datasets.Value("int32"),
                    "url": datasets.Value("string"),
                    "sentence_type": datasets.Value("string"),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage="http://rtw.ml.cmu.edu/rtw/",
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        my_urls = _URLs[self.config.name]
        data_dir = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples from the NELL belief knowledge base and candidate bleifs knowledge base if the config is 'nell_belief' and 'nell_candidate', respectively, otherwise yields the sentences for two dataset if the config is 'nell_belief_sentences' and 'nell_candidate_sentences' respectively."""

        with open(filepath, encoding="utf-8") as f:
            id_ = -1
            for row in f:
                row = row.strip().split("\t")
                if "[" in row[3]:
                    row[3] = row[3].strip("[]").split(",")[0]
                if "[" in row[4]:
                    row[4] = row[4].strip("[]").split(",")[0]
                if self.config.name in ("nell_belief", "nell_candidate"):
                    id_ += 1
                    yield id_, {
                        "entity": row[0].strip(),
                        "relation": row[1].strip(),
                        "value": row[2].strip(),
                        "iteration_of_promotion": row[3].strip(),
                        "score": row[4].strip(),
                        "source": row[5].strip(),
                        "entity_literal_strings": row[6].strip(),
                        "value_literal_strings": row[7].strip(),
                        "best_entity_literal_string": row[8].strip(),
                        "best_value_literal_string": row[9].strip(),
                        "categories_for_entity": row[10].strip(),
                        "categories_for_value": row[11].strip(),
                        "candidate_source": row[12].strip(),
                    }
                else:
                    best_arg1 = row[8]
                    best_arg2 = row[9]
                    iter_type = ""
                    for s2 in unquote(row[12]).strip("[]").split("-Iter"):
                        if iter_type in ("CPL", "OE"):
                            arr = unescape(s2.split(">", 1)[-1].strip("-").replace("+", " ")).split("\t")
                            la = len(arr)
                            count = 0
                            url = ""
                            for i in range(0, la, 2):
                                sentence = arr[i]
                                if i + 1 == la:
                                    count = 1
                                    url = ""
                                else:
                                    try:
                                        count = int(arr[i + 1].split(",")[0])
                                        url = ""
                                    except ValueError:
                                        count = 1
                                        url = ""
                                        if arr[i + 1].startswith("http"):
                                            url = arr[i + 1].split(",")[0]
                                if iter_type == "CPL":
                                    if "_" in sentence:
                                        sentence = sentence.replace("_", "[[ " + best_arg1 + " ]]")
                                    elif "arg1" in sentence:
                                        sentence = sentence.replace("arg1", "[[ " + best_arg1 + " ]]").replace(
                                            "arg2", "[[ " + best_arg2 + " ]]"
                                        )
                                    else:
                                        continue
                                if sentence.endswith("CPL"):
                                    sentence = sentence[:-5]
                                if sentence.endswith("OE"):
                                    sentence = sentence[:-4]
                                id_ += 1
                                yield id_, {
                                    "entity": row[0].replace("candidate:", "").replace("concept:", "").strip(),
                                    "relation": row[1].replace("candidate:", "").replace("concept:", "").strip(),
                                    "value": row[2].replace("candidate:", "").replace("concept:", "").strip(),
                                    "score": row[4].strip(),
                                    "sentence": sentence.strip(),
                                    "count": int(count),
                                    "url": url.strip(),
                                    "sentence_type": iter_type,
                                }
                        iter_type = s2.split(",")[-1].strip("+")



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Telugu Books Dataset"""


import csv
import os

import datasets


_CITATION = """\
@InProceedings{huggingface:dataset,
title = {Indic NLP - Natural Language Processing for Indian Languages},
authors = {Sudalai Rajkumar, Anusha Motamarri},
year={2019}
}
"""

_DESCRIPTION = """\
This dataset is created by scraping telugu novels from teluguone.com this dataset can be used for nlp tasks like topic modeling, word embeddings, transfer learning etc
"""

_HOMEPAGE = "https://www.kaggle.com/sudalairajkumar/telugu-nlp"

_LICENSE = "Data files © Original Authors"

_FILENAME = "telugu_books.csv"


class TeluguBooks(datasets.GeneratorBasedBuilder):
    """Telugu novels"""

    VERSION = datasets.Version("1.1.0")

    @property
    def manual_download_instructions(self):
        return """\
    You need to go to https://www.kaggle.com/sudalairajkumar/telugu-nlp,
    and manually download the telugu_books. Once it is completed,
    a file named telugu_books.zip will be appeared in your Downloads folder
    or whichever folder your browser chooses to save files to. You then have
    to unzip the file and move telugu_books,csv under <path/to/folder>.
    The <path/to/folder> can e.g. be "~/manual_data".
    telugu_books can then be loaded using the following command `datasets.load_dataset("telugu_books", data_dir="<path/to/folder>")`.
    """

    def _info(self):
        features = datasets.Features(
            {
                "text": datasets.Value("string"),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        path_to_manual_file = os.path.abspath(os.path.expanduser(dl_manager.manual_dir))
        if not os.path.exists(path_to_manual_file):
            raise FileNotFoundError(
                f"{path_to_manual_file} does not exist. Make sure you insert a manual dir via `datasets.load_dataset('telugu_books', data_dir=...)` that includes file name {_FILENAME}. Manual download instructions: {self.manual_download_instructions}"
            )
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(path_to_manual_file, "telugu_books.csv"),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(csv_file)
            for id_, row in enumerate(csv_reader):
                _, text = row
                yield id_, {"text": text}
